import pytest

from diofant import (Add, And, Ci, Derivative, DiracDelta, E, Eq, EulerGamma,
                     Expr, Function, I, Integral, Interval, Lambda, LambertW,
                     Matrix, Max, Min, Ne, O, Piecewise, Poly, Rational, Si,
                     Sum, Symbol, Tuple, acos, acosh, asin, asinh, atan, cbrt,
                     cos, cosh, diff, erf, erfi, exp, expand_func, expand_mul,
                     factor, fresnels, gamma, im, integrate, log, lowergamma,
                     meijerg, nan, oo, pi, polar_lift, polygamma, re, sign,
                     simplify, sin, sinh, sqrt, sstr, symbols, sympify, tan,
                     tanh, trigsimp)
from diofant.abc import A, L, R, a, b, c, h, i, k, m, s, t, w, x, y, z
from diofant.functions.elementary.complexes import periodic_argument
from diofant.integrals.heurisch import heurisch
from diofant.integrals.risch import NonElementaryIntegral
from diofant.utilities.randtest import verify_numerically


__all__ = ()

x_1, x_2 = symbols('x_1 x_2')
n = Symbol('n', integer=True)
f = Function('f')


def diff_test(i):
    """Return the set of symbols, s, which were used in testing that
    i.diff(s) agrees with i.doit().diff(s). If there is an error then
    the assertion will fail, causing the test to fail.
    """
    syms = i.free_symbols
    for s in syms:
        assert (i.diff(s).doit() - i.doit().diff(s)).expand() == 0
    return syms


def test_improper_integral():
    assert integrate(log(x), (x, 0, 1)) == -1
    assert integrate(x**(-2), (x, 1, oo)) == 1

    # issue sympy/sympy#10445:
    assert integrate(1/(1 + exp(x)), (x, 0, oo)) == log(2)


def test_constructor():
    # this is shared by Sum, so testing Integral's constructor
    # is equivalent to testing Sum's
    s1 = Integral(n, n)
    assert s1.limits == (Tuple(n),)
    s2 = Integral(n, (n,))
    assert s2.limits == (Tuple(n),)
    s3 = Integral(Sum(x, (x, 1, y)))
    assert s3.limits == (Tuple(y),)
    s4 = Integral(n, Tuple(n,))
    assert s4.limits == (Tuple(n),)

    s5 = Integral(n, (n, Interval(1, 2)))
    assert s5.limits == (Tuple(n, 1, 2),)


def test_basics():
    assert Integral(0, x) != 0
    assert Integral(x, (x, 1, 1)) != 0
    assert Integral(oo, x) != oo
    assert Integral(nan, x) == nan

    assert diff(Integral(y, y), x) == 0
    assert diff(Integral(x, (x, 0, 1)), x) == 0
    assert diff(Integral(x, x), x) == x
    assert diff(Integral(t, (t, 0, x)), x) == x + Integral(0, (t, 0, x))

    e = (t + 1)**2
    assert diff(integrate(e, (t, 0, x)), x) == \
        diff(Integral(e, (t, 0, x)), x).doit().expand() == \
        ((1 + x)**2).expand()
    assert diff(integrate(e, (t, 0, x)), t) == \
        diff(Integral(e, (t, 0, x)), t) == 0
    assert diff(integrate(e, (t, 0, x)), a) == \
        diff(Integral(e, (t, 0, x)), a) == 0
    assert diff(integrate(e, t), a) == diff(Integral(e, t), a) == 0

    assert integrate(e, (t, a, x)).diff(x) == \
        Integral(e, (t, a, x)).diff(x).doit().expand()
    assert Integral(e, (t, a, x)).diff(x).doit() == ((1 + x)**2)
    assert integrate(e, (t, x, a)).diff(x).doit() == (-(1 + x)**2).expand()

    assert integrate(t**2, (t, x, 2*x)).diff(x) == 7*x**2

    assert Integral(x, x).atoms() == {x}
    assert Integral(f(x), (x, 0, 1)).atoms() == {0, 1, x}

    assert diff_test(Integral(x, (x, 3*y))) == {y}
    assert diff_test(Integral(x, (a, 3*y))) == {x, y}

    assert integrate(x, (x, oo, oo)) == 0  # issue sympy/sympy#8171
    assert integrate(x, (x, -oo, -oo)) == 0

    # sum integral of terms
    assert integrate(y + x + exp(x), x) == x*y + x**2/2 + exp(x)

    assert Integral(x).is_commutative
    n = Symbol('n', commutative=False)
    assert Integral(n + x, x).is_commutative is False


def test_diff_wrt():
    class Test(Expr):
        _diff_wrt = True
        is_commutative = True

    t = Test()
    assert integrate(t + 1, t) == t**2/2 + t
    assert integrate(t + 1, (t, 0, 1)) == Rational(3, 2)

    pytest.raises(ValueError, lambda: integrate(x + 1, x + 1))
    pytest.raises(ValueError, lambda: integrate(x + 1, (x + 1, 0, 1)))


def test_basics_multiple():
    assert diff_test(Integral(x, (x, 3*x, 5*y), (y, x, 2*x))) == {x}
    assert diff_test(Integral(x, (x, 5*y), (y, x, 2*x))) == {x}
    assert diff_test(Integral(x, (x, 5*y), (y, y, 2*x))) == {x, y}
    assert diff_test(Integral(y, y, x)) == {x, y}
    assert diff_test(Integral(y*x, x, y)) == {x, y}
    assert diff_test(Integral(x + y, y, (y, 1, x))) == {x}
    assert diff_test(Integral(x + y, (x, x, y), (y, y, x))) == {x, y}


def test_conjugate_transpose():
    A, B = symbols('A B', commutative=False)

    x = Symbol('x', complex=True)
    p = Integral(A*B, (x,))
    assert p.adjoint().doit() == p.doit().adjoint()
    assert p.conjugate().doit() == p.doit().conjugate()
    assert p.transpose().doit() == p.doit().transpose()

    x = Symbol('x', extended_real=True)
    p = Integral(A*B, (x,))
    assert p.adjoint().doit() == p.doit().adjoint()
    assert p.conjugate().doit() == p.doit().conjugate()
    assert p.transpose().doit() == p.doit().transpose()


def test_integration():
    assert integrate(0, (t, 0, x)) == 0
    assert integrate(3, (t, 0, x)) == 3*x
    assert integrate(t, (t, 0, x)) == x**2/2
    assert integrate(3*t, (t, 0, x)) == 3*x**2/2
    assert integrate(3*t**2, (t, 0, x)) == x**3
    assert integrate(1/t, (t, 1, x)) == log(x)
    assert integrate(-1/t**2, (t, 1, x)) == 1/x - 1
    assert integrate(t**2 + 5*t - 8, (t, 0, x)) == x**3/3 + 5*x**2/2 - 8*x
    assert integrate(x**2, x) == x**3/3
    assert integrate((3*t*x)**5, x) == (3*t)**5 * x**6 / 6

    b = Symbol('b')
    c = Symbol('c')
    assert integrate(a*t, (t, 0, x)) == a*x**2/2
    assert integrate(a*t**4, (t, 0, x)) == a*x**5/5
    assert integrate(a*t**2 + b*t + c, (t, 0, x)) == a*x**3/3 + b*x**2/2 + c*x


def test_multiple_integration():
    assert integrate((x**2)*(y**2), (x, 0, 1), (y, -1, 2)) == 1
    assert integrate((y**2)*(x**2), x, y) == Rational(1, 9)*(x**3)*(y**3)
    assert integrate(1/(x + 3)/(1 + x)**3, x) == \
        -Rational(1, 8)*log(3 + x) + Rational(1, 8)*log(1 + x) + x/(4 + 8*x + 4*x**2)


def test_sympyissue_3532():
    assert integrate(exp(-x), (x, 0, oo)) == 1


def test_sympyissue_3560():
    assert integrate(sqrt(x)**3, x) == 2*sqrt(x)**5/5
    assert integrate(sqrt(x), x) == 2*sqrt(x)**3/3
    assert integrate(1/sqrt(x)**3, x) == -2/sqrt(x)


def test_integrate_poly():
    p = Poly(x + x**2*y + y**3, x, y)

    qx = integrate(p, x)
    qy = integrate(p, y)

    assert isinstance(qx, Poly) is True
    assert isinstance(qy, Poly) is True

    assert qx.gens == (x, y)
    assert qy.gens == (x, y)

    assert qx.as_expr() == x**2/2 + x**3*y/3 + x*y**3
    assert qy.as_expr() == x*y + x**2*y**2/2 + y**4/4


def test_integrate_poly_defined():
    p = Poly(x + x**2*y + y**3, x, y)

    Qx = integrate(p, (x, 0, 1))
    Qy = integrate(p, (y, 0, pi))

    assert isinstance(Qx, Poly) is True
    assert isinstance(Qy, Poly) is True

    assert Qx.gens == (y,)
    assert Qy.gens == (x,)

    assert Qx.as_expr() == Rational(1, 2) + y/3 + y**3
    assert Qy.as_expr() == pi**4/4 + pi*x + pi**2*x**2/2


def test_integrate_omit_var():
    assert integrate(x) == x**2/2

    pytest.raises(ValueError, lambda: integrate(2))
    pytest.raises(ValueError, lambda: integrate(x*y))


def test_integrate_poly_accurately():
    assert integrate(x*sin(y), x) == x**2*sin(y)/2

    # when passed to risch_norman, this will be a CPU hog, so this really
    # checks, that integrated function is recognized as polynomial
    assert integrate(x**1000*sin(y), x) == x**1001*sin(y)/1001


def test_sympyissue_3635():
    assert integrate(x**2, y) == x**2*y
    assert integrate(x**2, (y, -1, 1)) == 2*x**2

# works in diofant and py.test but hangs in `setup.py test`


def test_integrate_linearterm_pow():
    # check integrate((a*x+b)^c, x)  --  issue sympy/sympy#3499
    y = Symbol('y', positive=True)
    # TODO: Remove conds='none' below, let the assumption take care of it.
    assert integrate(x**y, x, conds='none') == x**(y + 1)/(y + 1)
    assert integrate((exp(y)*x + 1/y)**(1 + sin(y)), x, conds='none') == \
        exp(-y)*(exp(y)*x + 1/y)**(2 + sin(y)) / (2 + sin(y))


def test_sympyissue_3618():
    assert integrate(pi*sqrt(x), x) == 2*pi*sqrt(x)**3/3
    assert integrate(pi*sqrt(x) + E*sqrt(x)**3, x) == \
        2*pi*sqrt(x)**3/3 + 2*E * sqrt(x)**5/5


def test_sympyissue_3623():
    assert integrate(cos((n + 1)*x), x) == Piecewise(
        (x, Eq(n + 1, 0)), (sin((n + 1)*x)/(n + 1), True))
    assert integrate(cos((n - 1)*x), x) == Piecewise(
        (x, Eq(n - 1, 0)), (sin((n - 1)*x)/(n - 1), True))
    assert integrate(cos((n + 1)*x) + cos((n - 1)*x), x) == \
        Piecewise((x, Eq(n + 1, 0)), (sin((n + 1)*x)/(n + 1), True)) + \
        Piecewise((x, Eq(n - 1, 0)), (sin((n - 1)*x)/(n - 1), True))


def test_sympyissue_3664():
    n = Symbol('n', integer=True, nonzero=True)
    assert integrate(-1./2 * x * sin(n * pi * x/2), [x, -2, 0]) == \
        2*cos(pi*n)/(pi*n)
    assert integrate(-x*sin(n*pi*x/2)/2, [x, -2, 0]) == \
        2*cos(pi*n)/(pi*n)


def test_sympyissue_3679():
    # definite integration of rational functions gives wrong answers
    assert NS(Integral(1/(x**2 - 8*x + 17), (x, 2, 4))) == '1.10714871779409'


def test_sympyissue_3686():  # remove this when fresnel itegrals are implemented
    assert expand_func(integrate(sin(x**2), x)) == \
        sqrt(2)*sqrt(pi)*fresnels(sqrt(2)*x/sqrt(pi))/2


def test_transcendental_functions():
    assert integrate(LambertW(2*x), x) == \
        -x + x*LambertW(2*x) + x/LambertW(2*x)


def test_sympyissue_3740():
    f = 4*log(x) - 2*log(x)**2
    fid = diff(integrate(f, x), x)
    assert abs(f.subs({x: 42}).evalf() - fid.subs({x: 42}).evalf()) < 1e-10


def test_sympyissue_3788():
    assert integrate(1/(1 + x**2), x) == atan(x)


def test_sympyissue_3952():
    f = sin(x)
    assert integrate(f, x) == -cos(x)
    pytest.raises(ValueError, lambda: integrate(f, 2*x))


def test_sympyissue_4516():
    assert integrate(2**x - 2*x, x) == 2**x/log(2) - x**2


def test_sympyissue_7450():
    ans = integrate(exp(-(1 + I)*x), (x, 0, oo))
    assert re(ans) == Rational(1, 2) and im(ans) == Rational(-1, 2)


def test_matrices():
    M = Matrix(2, 2, lambda i, j: (i + j + 1)*sin((i + j + 1)*x))

    assert integrate(M, x) == Matrix([
        [-cos(x), -cos(2*x)],
        [-cos(2*x), -cos(3*x)],
    ])


def test_integrate_functions():
    # issue sympy/sympy#4111
    assert integrate(f(x), x) == Integral(f(x), x)
    assert integrate(f(x), (x, 0, 1)) == Integral(f(x), (x, 0, 1))

    assert integrate(Derivative(f(y), y), x) == x*Derivative(f(y), y)


@pytest.mark.xfail
def test_integrate_functions_1():
    assert integrate(f(x)*diff(f(x), x), x) == f(x)**2/2
    assert integrate(diff(f(x), x)/f(x), x) == log(f(x))


@pytest.mark.xfail
def test_integrate_derivatives():
    assert integrate(Derivative(f(x), x), x) == f(x)


def test_transform():
    a = Integral(x**2 + 1, (x, -1, 2))
    fx = x
    fy = 3*y + 1
    assert a.doit() == a.transform(fx, fy).doit()
    assert a.transform(fx, fy).transform(fy, fx) == a
    fx = 3*x + 1
    fy = y
    assert a.transform(fx, fy).transform(fy, fx) == a
    a = Integral(sin(1/x), (x, 0, 1))
    assert a.transform(x, 1/y) == Integral(sin(y)/y**2, (y, 1, oo))
    assert a.transform(x, 1/y).transform(y, 1/x) == a
    a = Integral(exp(-x**2), (x, -oo, oo))
    assert a.transform(x, 2*y) == Integral(2*exp(-4*y**2), (y, -oo, oo))
    # < 3 arg limit handled properly
    assert Integral(x, x).transform(x, a*y).doit() == \
        Integral(y*a**2, y).doit()
    assert Integral(x, (x, 0, -3)).transform(x, 1/y).doit() == \
        Integral(-1/x**3, (x, -oo, Rational(-1, 3))).doit()
    assert Integral(x, (x, 0, 3)).transform(x, 1/y) == \
        Integral(y**(-3), (y, Rational(1, 3), oo))
    # issue sympy/sympy#8400
    i = Integral(x + y, (x, 1, 2), (y, 1, 2))
    assert i.transform(x, (x + 2*y, x)).doit() == \
        i.transform(x, (x + 2*z, x)).doit() == 3

    assert a.transform(b, x) == a
    assert a.transform(x, y) == Integral(exp(-y**2), (y, -oo, oo))

    i2 = Integral(cos(x**2 - 1), (x, 0, y))
    i = i2.subs({y: 1})

    pytest.raises(ValueError, lambda: i.transform(x**2 - 1, y))
    pytest.raises(ValueError, lambda: i.transform(x, y*z))
    pytest.raises(ValueError, lambda: i.transform(x, (y, y + z)))
    pytest.raises(ValueError, lambda: i2.transform(x, (z*y, y)))
    pytest.raises(ValueError, lambda: i.transform(x, (sin(y), y)))


def test_sympyissue_4052():
    f = Rational(1, 2)*asin(x) + x*sqrt(1 - x**2)/2

    assert integrate(cos(asin(x)), x) == f
    assert integrate(sin(acos(x)), x) == f


def NS(e, n=15, **options):
    return sstr(sympify(e).evalf(n, **options), full_prec=True)


def test_evalf_integrals():
    assert NS(Integral(cos(x)/x, (x, 1, oo)), quad='osc') == '-0.337403922900968'

    pytest.raises(ValueError,
                  lambda: NS(Integral(acos(x)/x, (x, 1, oo)), quad='osc'))

    assert NS(Integral(sin(x + I), (x, 0, pi/2))) == '1.54308063481524 + 1.17520119364380*I'

    assert Integral(pi, (x, y, z)).evalf() == Integral(pi, (x, y, z))
    assert Integral(pi, (x, y, y + z)).evalf() == Integral(pi, (x, y, y + z))
    #
    # Endpoints causing trouble (rounding error in integration points -> complex log)
    assert NS(
        2 + Integral(log(2*cos(x/2)), (x, -pi, pi)), 17, chop=True) == NS(2, 17)
    assert NS(
        2 + Integral(log(2*cos(x/2)), (x, -pi, pi)), 20, chop=True) == NS(2, 20)
    assert NS(
        2 + Integral(log(2*cos(x/2)), (x, -pi, pi)), 22, chop=True) == NS(2, 22)


@pytest.mark.slow
def test_evalf_integrals_slow():
    assert NS(Integral(x, (x, 2, 5)), 15) == '10.5000000000000'
    gauss = Integral(exp(-x**2), (x, -oo, oo))
    assert NS(gauss, 15) == '1.77245385090552'
    assert NS(gauss**2 - pi + E*Rational(
        1, 10**20), 15) in ('2.71828182845904e-20', '2.71828182845905e-20')
    # A monster of an integral from https://mathworld.wolfram.com/DefiniteIntegral.html
    a = 8*sqrt(3)/(1 + 3*t**2)
    b = 16*sqrt(2)*(3*t + 1)*sqrt(4*t**2 + t + 1)**3
    c = (3*t**2 + 1)*(11*t**2 + 2*t + 3)**2
    d = sqrt(2)*(249*t**2 + 54*t + 65)/(11*t**2 + 2*t + 3)**2
    f = a - b/c - d
    assert NS(Integral(f, (t, 0, 1)), 50) == \
        NS((3*sqrt(2) - 49*pi + 162*atan(sqrt(2)))/12, 50)
    # https://mathworld.wolfram.com/VardisIntegral.html
    assert NS(Integral(log(log(1/x))/(1 + x + x**2), (x, 0, 1)), 15) == \
        NS('pi/sqrt(3) * log(2*pi**(5/6) / gamma(1/6))', 15)
    # https://mathworld.wolfram.com/AhmedsIntegral.html
    assert NS(Integral(atan(sqrt(x**2 + 2))/(sqrt(x**2 + 2)*(x**2 + 1)), (x,
                                                                          0, 1)), 15) == NS(5*pi**2/96, 15)
    # https://mathworld.wolfram.com/AbelsIntegral.html
    assert NS(Integral(x/((exp(pi*x) - exp(
        -pi*x))*(x**2 + 1)), (x, 0, oo)), 15) == NS('log(2)/2-1/4', 15)
    # Complex part trimming
    # https://mathworld.wolfram.com/VardisIntegral.html
    assert NS(Integral(log(log(sin(x)/cos(x))), (x, pi/4, pi/2)), 15, chop=True) == \
        NS('pi/4*log(4*pi**3/gamma(1/4)**4)', 15)
    # Needs zero handling
    assert NS(pi - 4*Integral(sqrt(1 - x**2), (x, 0, 1)),
              15, maxn=30, chop=True, strict=False) in ('0.0', '0')
    # Oscillatory quadrature
    a = Integral(sin(x)/x**2, (x, 1, oo)).evalf(maxn=15, strict=False)
    assert 0.49 < a < 0.51
    assert NS(
        Integral(sin(x)/x**2, (x, 1, oo)), quad='osc') == '0.504067061906928'
    assert NS(Integral(
        cos(pi*x + 1)/x, (x, -oo, -1)), quad='osc') == '0.276374705640365'
    # indefinite integrals aren't evaluated
    assert NS(Integral(x, x)) == 'Integral(x, x)'
    assert NS(Integral(x, (x, y))) == 'Integral(x, (x, y))'


def test_evalf_sympyissue_4038():
    # The output form of an integral may differ by a step function between
    # revisions, making this test a bit useless. This can't be said about
    # other two tests. For now, all values of this evaluation are used here,
    # but in future this should be reconsidered.
    assert NS(integrate(1/(x**5 + 1), x).subs({x: 4}), chop=True) in \
        ['-0.000976138910649103', '0.965906660135753', '1.93278945918216']

    assert NS(Integral(1/(x**5 + 1), (x, 2, 4))) == '0.0144361088886740'
    assert NS(
        integrate(1/(x**5 + 1), (x, 2, 4)), chop=True) == '0.0144361088886740'


@pytest.mark.xfail
def test_failing_integrals():
    assert isinstance(NS(Integral(sqrt(x) + x*y, (x, 1, 2), (y, -1, 1))),
                      Float)  # == '2.43790283299492'
    assert isinstance(NS(Integral(sin(x + x*y), (x, -1, 1), (y, -1, 1))),
                      Float)  # == '0.0'


def test_integrate_DiracDelta():
    # This is here to check that deltaintegrate is being called, but also
    # to test definite integrals. More tests are in test_deltafunctions.py
    assert integrate(DiracDelta(x) * f(x), (x, -oo, oo)) == f(0)
    assert integrate(DiracDelta(x) * f(x), (x, 0, oo)) == f(0)/2
    assert integrate(DiracDelta(x)**2, (x, -oo, oo)) == DiracDelta(0)
    # issue sympy/sympy#4522
    assert integrate(integrate((4 - 4*x + x*y - 4*y) *
                               DiracDelta(x)*DiracDelta(y - 1), (x, 0, 1)), (y, 0, 1)) == 0
    # issue sympy/sympy#5729
    p = exp(-(x**2 + y**2))/pi
    assert integrate(p*DiracDelta(x - 10*y), (x, -oo, oo), (y, -oo, oo)) == \
        integrate(p*DiracDelta(x - 10*y), (y, -oo, oo), (x, -oo, oo)) == \
        integrate(p*DiracDelta(10*x - y), (x, -oo, oo), (y, -oo, oo)) == \
        integrate(p*DiracDelta(10*x - y), (y, -oo, oo), (x, -oo, oo)) == \
        1/sqrt(101*pi)


@pytest.mark.xfail
@pytest.mark.slow
def test_integrate_DiracDelta_fails():
    # issue sympy/sympy#6427
    integrate(DiracDelta(x - y - z), (x, 0, 1),
              (y, 0, 1), (z, 0, oo))  # = Rational(1, 2)


def test_integrate_returns_piecewise():
    assert integrate(x**y, x) == Piecewise(
        (log(x), Eq(y, -1)), (x**(y + 1)/(y + 1), True))
    assert integrate(x**y, y) == Piecewise(
        (y, Eq(log(x), 0)), (x**y/log(x), True))
    assert integrate(exp(n*x), x) == Piecewise(
        (x, Eq(n, 0)), (exp(n*x)/n, True))
    assert integrate(x*exp(n*x), x) == Piecewise(
        (x**2/2, Eq(n**3, 0)), ((x*n**2 - n)*exp(n*x)/n**3, True))
    assert integrate(x**(n*y), x) == Piecewise(
        (log(x), Eq(n*y, -1)), (x**(n*y + 1)/(n*y + 1), True))
    assert integrate(x**(n*y), y) == Piecewise(
        (y, Eq(n*log(x), 0)), (x**(n*y)/(n*log(x)), True))
    assert integrate(cos(n*x), x) == Piecewise(
        (x, Eq(n, 0)), (sin(n*x)/n, True))
    assert integrate(cos(n*x)**2, x) == Piecewise(
        (x, Eq(n, 0)), ((n*x/2 + sin(n*x)*cos(n*x)/2)/n, True))
    assert integrate(x*cos(n*x), x) == Piecewise(
        (x**2/2, Eq(n, 0)), (x*sin(n*x)/n + cos(n*x)/n**2, True))
    assert integrate(sin(n*x), x) == Piecewise(
        (0, Eq(n, 0)), (-cos(n*x)/n, True))
    assert integrate(sin(n*x)**2, x) == Piecewise(
        (0, Eq(n, 0)), ((n*x/2 - sin(n*x)*cos(n*x)/2)/n, True))
    assert integrate(x*sin(n*x), x) == Piecewise(
        (0, Eq(n, 0)), (-x*cos(n*x)/n + sin(n*x)/n**2, True))
    assert integrate(exp(x*y), (x, 0, z)) == Piecewise(
        (z, Eq(y, 0)), (exp(y*z)/y - 1/y, True))


def test_subs1():
    e = Integral(exp(x - y), x)
    assert e.subs({y: 3}) == Integral(exp(x - 3), x)
    e = Integral(exp(x - y), (x, 0, 1))
    assert e.subs({y: 3}) == Integral(exp(x - 3), (x, 0, 1))
    f = Lambda(x, exp(-x**2))
    conv = Integral(f(x - y)*f(y), (y, -oo, oo))
    assert conv.subs({x: 0}) == Integral(exp(-2*y**2), (y, -oo, oo))


def test_subs2():
    e = Integral(exp(x - y), x, t)
    assert e.subs({y: 3}) == Integral(exp(x - 3), x, t)
    e = Integral(exp(x - y), (x, 0, 1), (t, 0, 1))
    assert e.subs({y: 3}) == Integral(exp(x - 3), (x, 0, 1), (t, 0, 1))
    f = Lambda(x, exp(-x**2))
    conv = Integral(f(x - y)*f(y), (y, -oo, oo), (t, 0, 1))
    assert conv.subs({x: 0}) == Integral(exp(-2*y**2), (y, -oo, oo), (t, 0, 1))


def test_subs3():
    e = Integral(exp(x - y), (x, 0, y), (t, y, 1))
    assert e.subs({y: 3}) == Integral(exp(x - 3), (x, 0, 3), (t, 3, 1))
    f = Lambda(x, exp(-x**2))
    conv = Integral(f(x - y)*f(y), (y, -oo, oo), (t, x, 1))
    assert conv.subs({x: 0}) == Integral(exp(-2*y**2), (y, -oo, oo), (t, 0, 1))


def test_subs4():
    e = Integral(exp(x), (x, 0, y), (t, y, 1))
    assert e.subs({y: 3}) == Integral(exp(x), (x, 0, 3), (t, 3, 1))
    f = Lambda(x, exp(-x**2))
    conv = Integral(f(y)*f(y), (y, -oo, oo), (t, x, 1))
    assert conv.subs({x: 0}) == Integral(exp(-2*y**2), (y, -oo, oo), (t, 0, 1))


def test_subs5():
    e = Integral(exp(-x**2), (x, -oo, oo))
    assert e.subs({x: 5}) == e
    e = Integral(exp(-x**2 + y), x)
    assert e.subs({y: 5}) == Integral(exp(-x**2 + 5), x)
    e = Integral(exp(-x**2 + y), (x, x))
    assert e.subs({x: 5}) == Integral(exp(y - x**2), (x, 5))
    assert e.subs({y: 5}) == Integral(exp(-x**2 + 5), x)
    e = Integral(exp(-x**2 + y), (y, -oo, oo), (x, -oo, oo))
    assert e.subs({x: 5}) == e
    assert e.subs({y: 5}) == e
    # Test evaluation of antiderivatives
    e = Integral(exp(-x**2), (x, x))
    assert e.subs({x: 5}) == Integral(exp(-x**2), (x, 5))
    e = Integral(exp(x), x)
    assert (e.subs({x: 1}) - e.subs({x: 0}) -
            Integral(exp(x), (x, 0, 1))).doit().is_zero


def test_subs6():
    e = Integral(x*y, (x, f(x), f(y)))
    assert e.subs({x: 1}) == Integral(x*y, (x, f(1), f(y)))
    assert e.subs({y: 1}) == Integral(x, (x, f(x), f(1)))
    e = Integral(x*y, (x, f(x), f(y)), (y, f(x), f(y)))
    assert e.subs({x: 1}) == Integral(x*y, (x, f(1), f(y)), (y, f(1), f(y)))
    assert e.subs({y: 1}) == Integral(x*y, (x, f(x), f(y)), (y, f(x), f(1)))
    e = Integral(x*y, (x, f(x), f(a)), (y, f(x), f(a)))
    assert e.subs({a: 1}) == Integral(x*y, (x, f(x), f(1)), (y, f(x), f(1)))


def test_subs7():
    e = Integral(x, (x, 1, y), (y, 1, 2))
    assert e.subs({x: 1, y: 2}) == e
    e = Integral(sin(x) + sin(y), (x, sin(x), sin(y)),
                 (y, 1, 2))
    assert e.subs({sin(y): 1}) == e
    assert e.subs({sin(x): 1}) == Integral(sin(x) + sin(y), (x, 1, sin(y)),
                                           (y, 1, 2))


def test_expand():
    e = Integral(f(x)+f(x**2), (x, 1, y))
    assert e.expand() == Integral(f(x), (x, 1, y)) + Integral(f(x**2), (x, 1, y))


def test_integration_variable():
    pytest.raises(ValueError, lambda: Integral(exp(-x**2), 3))
    pytest.raises(ValueError, lambda: Integral(exp(-x**2), (3, -oo, oo)))


def test_expand_integral():
    assert Integral(cos(x**2)*(sin(x**2) + 1), (x, 0, 1)).expand() == \
        Integral(cos(x**2)*sin(x**2), (x, 0, 1)) + \
        Integral(cos(x**2), (x, 0, 1))
    assert Integral(cos(x**2)*(sin(x**2) + 1), x).expand() == \
        Integral(cos(x**2)*sin(x**2), x) + \
        Integral(cos(x**2), x)


def test_as_sum_midpoint1():
    e = Integral(sqrt(x**3 + 1), (x, 2, 10))
    assert e.as_sum(1, method='midpoint') == 8*sqrt(217)
    assert e.as_sum(2, method='midpoint') == 4*sqrt(65) + 12*sqrt(57)
    assert e.as_sum(3, method='midpoint') == 8*sqrt(217)/3 + \
        8*sqrt(3081)/27 + 8*sqrt(52809)/27
    assert e.as_sum(4, method='midpoint') == 2*sqrt(730) + \
        4*sqrt(7) + 4*sqrt(86) + 6*sqrt(14)
    assert abs(e.as_sum(4, method='midpoint').evalf() - e.evalf()) < 0.5

    e = Integral(sqrt(x**3 + y**3), (x, 2, 10), (y, 0, 10))
    pytest.raises(NotImplementedError, lambda: e.as_sum(4))


def test_as_sum_midpoint2():
    e = Integral((x + y)**2, (x, 0, 1))
    assert e.as_sum(1, method='midpoint').expand() == Rational(1, 4) + y + y**2
    assert e.as_sum(2, method='midpoint').expand() == Rational(5, 16) + y + y**2
    assert e.as_sum(3, method='midpoint').expand() == Rational(35, 108) + y + y**2
    assert e.as_sum(4, method='midpoint').expand() == Rational(21, 64) + y + y**2


def test_as_sum_trapezoid():
    e = Integral(sin(x), (x, 3, 7))
    assert e.as_sum(2, 'trapezoid') == 2*sin(5) + sin(3) + sin(7)


def test_as_sum_left():
    e = Integral((x + y)**2, (x, 0, 1))
    assert e.as_sum(1, method='left').expand() == y**2
    assert e.as_sum(2, method='left').expand() == Rational(1, 8) + y/2 + y**2
    assert e.as_sum(3, method='left').expand() == Rational(5, 27) + 2*y/3 + y**2
    assert e.as_sum(4, method='left').expand() == Rational(7, 32) + 3*y/4 + y**2


def test_as_sum_right():
    e = Integral((x + y)**2, (x, 0, 1))
    assert e.as_sum(1, method='right').expand() == 1 + 2*y + y**2
    assert e.as_sum(2, method='right').expand() == Rational(5, 8) + 3*y/2 + y**2
    assert e.as_sum(3, method='right').expand() == Rational(14, 27) + 4*y/3 + y**2
    assert e.as_sum(4, method='right').expand() == Rational(15, 32) + 5*y/4 + y**2


def test_as_sum_raises():
    e = Integral((x + y)**2, (x, 0, 1))
    pytest.raises(ValueError, lambda: e.as_sum(-1))
    pytest.raises(ValueError, lambda: e.as_sum(0))
    pytest.raises(ValueError, lambda: Integral(x).as_sum(3))
    pytest.raises(NotImplementedError, lambda: e.as_sum(oo))
    pytest.raises(NotImplementedError, lambda: e.as_sum(3, method='xxxx2'))


def test_integrate_conds():
    assert integrate(x**a*exp(-x), (x, 0, oo),
                     conds='separate') == (gamma(a + 1), -re(a) < 1)


def test_nested_doit():
    e = Integral(Integral(x, x), x)
    f = Integral(x, x, x)
    assert e.doit() == f.doit()


def test_sympyissue_4665():
    # Allow only upper or lower limit evaluation
    e = Integral(x**2, (x, None, 1))
    f = Integral(x**2, (x, 1, None))
    assert e.doit() == Rational(1, 3)
    assert f.doit() == Rational(-1, 3)
    assert Integral(x*y, (x, None, y)).subs({y: t}) == Integral(x*t, (x, None, t))
    assert Integral(x*y, (x, y, None)).subs({y: t}) == Integral(x*t, (x, t, None))
    assert integrate(x**2, (x, None, 1)) == Rational(1, 3)
    assert integrate(x**2, (x, 1, None)) == Rational(-1, 3)
    assert integrate('x**2', ('x', '1', None)) == Rational(-1, 3)


def test_integral_reconstruct():
    e = Integral(x**2, (x, -1, 1))
    assert e == Integral(*e.args)


def test_doit_integrals():
    e = Integral(Integral(2*x), (x, 0, 1))
    assert e.doit() == Rational(1, 3)
    assert e.doit(deep=False) == Rational(1, 3)
    f = Function('f')
    # doesn't matter if the integral can't be performed
    assert Integral(f(x), (x, 1, 1)).doit() == 0
    # doesn't matter if the limits can't be evaluated
    assert Integral(0, (x, 1, Integral(f(x), x))).doit() == 0
    assert Integral(x, (a, 0)).doit() == 0
    limits = ((a, 1, exp(x)), (x, 0))
    assert Integral(a, *limits).doit() == Rational(1, 4)
    assert Integral(a, *list(reversed(limits))).doit() == 0


def test_as_dummy():
    assert Integral(x, x).as_dummy() == Integral(x, x)


def test_sympyissue_4884():
    assert integrate(sqrt(x)*(1 + x)) == \
        Piecewise(
            (2*sqrt(x)*(x + 1)**2/5 - 2*sqrt(x)*(x + 1)/15 - 4*sqrt(x)/15,
             abs(x + 1) > 1),
            (2*I*sqrt(-x)*(x + 1)**2/5 - 2*I*sqrt(-x)*(x + 1)/15 -
             4*I*sqrt(-x)/15, True))
    assert integrate(x**x*(1 + log(x))) == x**x


def test_is_number():
    assert Integral(x).is_number is False
    assert Integral(1, x).is_number is False
    assert Integral(1, (x, 1)).is_number is True
    assert Integral(1, (x, 1, 2)).is_number is True
    assert Integral(1, (x, 1, y)).is_number is False
    assert Integral(1, (x, y)).is_number is False
    assert Integral(x, y).is_number is False
    assert Integral(x, (y, 1, x)).is_number is False
    assert Integral(x, (y, 1, 2)).is_number is False
    assert Integral(x, (x, 1, 2)).is_number is True
    # `foo.is_number` should always be eqivalent to `not foo.free_symbols`
    # in each of these cases, there are pseudo-free symbols
    i = Integral(x, (y, 1, 1))
    assert i.is_number is False and i.evalf() == 0
    i = Integral(x, (y, z, z))
    assert i.is_number is False and i.evalf() == 0
    i = Integral(1, (y, z, z + 2))
    assert i.is_number is False and i.evalf() == 2

    assert Integral(x*y, (x, 1, 2), (y, 1, 3)).is_number is True
    assert Integral(x*y, (x, 1, 2), (y, 1, z)).is_number is False
    assert Integral(x, (x, 1)).is_number is True
    assert Integral(x, (x, 1, Integral(y, (y, 1, 2)))).is_number is True
    assert Integral(Sum(z, (z, 1, 2)), (x, 1, 2)).is_number is True
    # it is possible to get a false negative if the integrand is
    # actually an unsimplified zero, but this is true of is_number in general.
    assert Integral(sin(x)**2 + cos(x)**2 - 1, x).is_number is False
    assert Integral(f(x), (x, 0, 1)).is_number is True


def test_symbols():
    assert Integral(0, x).free_symbols == {x}
    assert Integral(x).free_symbols == {x}
    assert Integral(x, (x, None, y)).free_symbols == {y}
    assert Integral(x, (x, y, None)).free_symbols == {y}
    assert Integral(x, (x, 1, y)).free_symbols == {y}
    assert Integral(x, (x, y, 1)).free_symbols == {y}
    assert Integral(x, (x, x, y)).free_symbols == {x, y}
    assert Integral(x, x, y).free_symbols == {x, y}
    assert Integral(x, (x, 1, 2)).free_symbols == set()
    assert Integral(x, (y, 1, 2)).free_symbols == {x}
    # pseudo-free in this case
    assert Integral(x, (y, z, z)).free_symbols == {x, z}
    assert Integral(x, (y, 1, 2), (y, None, None)).free_symbols == {x, y}
    assert Integral(x, (y, 1, 2), (x, 1, y)).free_symbols == {y}
    assert Integral(2, (y, 1, 2), (y, 1, x), (x, 1, 2)).free_symbols == set()
    assert Integral(2, (y, x, 2), (y, 1, x), (x, 1, 2)).free_symbols == set()
    assert Integral(2, (x, 1, 2), (y, x, 2), (y, 1, 2)).free_symbols == \
        {x}


def test_is_zero():
    assert Integral(0, (x, 1, x)).is_zero
    assert Integral(1, (x, 1, 1)).is_zero
    assert Integral(1, (x, m)).is_zero is None
    assert Integral(1, (x, 1, 2), (y, 2)).is_nonzero
    assert Integral(x, (m, 0)).is_zero
    assert Integral(x + m, (m, 0)).is_zero is None
    i = Integral(m, (m, 1, exp(x)), (x, 0))
    assert i.is_zero is None
    assert Integral(m, (x, 0), (m, 1, exp(x))).is_zero

    assert Integral(x, (x, oo, oo)).is_zero  # issue sympy/sympy#8171
    assert Integral(x, (x, -oo, -oo)).is_zero

    # this is zero but is beyond the scope of what is_zero
    # should be doing
    assert Integral(sin(x), (x, 0, 2*pi)).is_zero is None


def test_is_real():
    assert Integral(x**3, (x, 1, 3)).is_real
    assert Integral(1/(x - 1), (x, -1, 1)).is_real is not True


def test_series():
    i = Integral(cos(x), (x, x))
    e = i.lseries(x)
    s1 = i.nseries(x, n=8).removeO().doit()
    s2 = Add(*[next(e) for j in range(4)])
    assert s1 == s2


def test_sympyissue_4403():
    x = Symbol('x')
    z = Symbol('z', positive=True)
    assert integrate(sqrt(x**2 + z**2), x) == \
        z**2*asinh(x/z)/2 + x*sqrt(x**2 + z**2)/2
    assert integrate(sqrt(x**2 - z**2), x) == \
        -z**2*acosh(x/z)/2 + x*sqrt(x**2 - z**2)/2

    x = Symbol('x', extended_real=True)
    y = Symbol('y', positive=True)
    assert integrate(1/(x**2 + y**2)**Rational(3, 2), x) == \
        x/(y**2*sqrt(x**2 + y**2))


def test_sympyissue_4403_2():
    assert integrate(sqrt(-x**2 - 4), x) == \
        -2*atan(x/sqrt(-4 - x**2)) + x*sqrt(-4 - x**2)/2


def test_sympyissue_4100():
    R = Symbol('R', positive=True)
    assert integrate(sqrt(R**2 - x**2), (x, 0, R)) == pi*R**2/4


def test_sympyissue_5167():
    f = Function('f')
    assert Integral(Integral(f(x), x), x) == Integral(f(x), x, x)
    assert Integral(f(x)).args == (f(x), Tuple(x))
    assert Integral(Integral(f(x))).args == (f(x), Tuple(x), Tuple(x))
    assert Integral(Integral(f(x)), y).args == (f(x), Tuple(x), Tuple(y))
    assert Integral(Integral(f(x), z), y).args == (f(x), Tuple(z), Tuple(y))
    assert Integral(Integral(Integral(f(x), x), y), z).args == \
        (f(x), Tuple(x), Tuple(y), Tuple(z))
    assert integrate(Integral(f(x), x), x) == Integral(f(x), x, x)
    assert integrate(Integral(f(x), y), x) == y*Integral(f(x), x)
    assert integrate(Integral(f(x), x), y) in [Integral(y*f(x), x), y*Integral(f(x), x)]
    assert integrate(Integral(2, x), x) == x**2
    assert integrate(Integral(2, x), y) == 2*x*y
    # don't re-order given limits
    assert Integral(1, x, y).args != Integral(1, y, x).args
    # do as many as possibble
    assert Integral(f(x), y, x, y, x).doit() == y**2*Integral(f(x), x, x)/2
    assert Integral(f(x), (x, 1, 2), (w, 1, x), (z, 1, y)).doit() == \
        y*(x - 1)*Integral(f(x), (x, 1, 2)) - (x - 1)*Integral(f(x), (x, 1, 2))


def test_sympyissue_4890():
    z = Symbol('z', positive=True)
    assert integrate(exp(-log(x)**2), x) == \
        sqrt(pi)*exp(Rational(1, 4))*erf(log(x)-Rational(1, 2))/2
    assert integrate(exp(log(x)**2), x) == \
        sqrt(pi)*exp(-Rational(1, 4))*erfi(log(x)+Rational(1, 2))/2
    assert integrate(exp(-z*log(x)**2), x) == \
        sqrt(pi)*exp(1/(4*z))*erf(sqrt(z)*log(x) - 1/(2*sqrt(z)))/(2*sqrt(z))


def test_sympyissue_4376():
    n = Symbol('n', integer=True, positive=True)
    assert simplify(integrate(n*(x**(1/n) - 1), (x, 0, Rational(1, 2))) -
                    (n**2 - 2**(1/n)*n**2 - n*2**(1/n))/(2**(1 + 1/n) + n*2**(1 + 1/n))) == 0


@pytest.mark.slow
def test_sympyissue_4517():
    assert integrate((sqrt(x) - x**3)/cbrt(x), x) == \
        6*x**Rational(7, 6)/7 - 3*x**Rational(11, 3)/11


def test_sympyissue_4527():
    k, m = symbols('k m', integer=True)
    assert integrate(sin(k*x)*sin(m*x), (x, 0, pi)) == Piecewise(
        (0, And(Eq(k, 0), Eq(m, 0))),
        (-pi/2, Eq(k, -m)),
        (pi/2, Eq(k, m)),
        (0, True))
    assert integrate(sin(k*x)*sin(m*x), (x,)) == Piecewise(
        (0, And(Eq(k, 0), Eq(m, 0))),
        (-x*sin(m*x)**2/2 - x*cos(m*x)**2/2 + sin(m*x)*cos(m*x)/(2*m), Eq(k, -m)),
        (x*sin(m*x)**2/2 + x*cos(m*x)**2/2 - sin(m*x)*cos(m*x)/(2*m), Eq(k, m)),
        (m*sin(k*x)*cos(m*x)/(k**2 - m**2) -
         k*sin(m*x)*cos(k*x)/(k**2 - m**2), True))


def test_sympyissue_4199():
    ypos = Symbol('y', positive=True)
    # TODO: Remove conds='none' below, let the assumption take care of it.
    assert (integrate(exp(-I*2*pi*ypos*x)*x, (x, -oo, oo), conds='none') ==
            Integral(exp(-I*2*pi*ypos*x)*x, (x, -oo, oo)))
    assert (integrate(exp(-I*2*pi*ypos*x)*x, (x, 0, oo), conds='none') ==
            Integral(exp(-2*I*pi*x*ypos)*x, (x, 0, oo)))


@pytest.mark.slow
def test_sympyissue_3940():
    a, b, c, d = symbols('a:d', positive=True, finite=True)
    assert integrate(exp(-x**2 + I*c*x), x) == \
        -sqrt(pi)*exp(-c**2/4)*erf(I*c/2 - x)/2
    assert integrate(exp(a*x**2 + b*x + c), x) == \
        sqrt(pi)*exp(c)*exp(-b**2/(4*a))*erfi(sqrt(a)*x + b/(2*sqrt(a)))/(2*sqrt(a))

    assert expand_mul(integrate(exp(-x**2)*exp(I*k*x), (x, -oo, oo))) == \
        sqrt(pi)*exp(-k**2/4)
    a, d = symbols('a d', positive=True)
    assert expand_mul(integrate(exp(-a*x**2 + 2*d*x), (x, -oo, oo))) == \
        sqrt(pi)*exp(d**2/a)/sqrt(a)


def test_sympyissue_5413():
    # Note that this is not the same as testing ratint() becuase integrate()
    # pulls out the coefficient.
    assert integrate(-a/(a**2 + x**2), x) == I*log(-I*a + x)/2 - I*log(I*a + x)/2


def test_sympyissue_5907():
    a = Symbol('a', real=True)
    assert (integrate(1/(x**2 + a**2)**2, x) ==
            x/(2*a**4 + 2*a**2*x**2) + atan(x/a)/(2*a**3))


def test_sympyissue_4892a():
    c = Symbol('c', nonzero=True)
    P1 = -A*exp(-z)
    P2 = -A/(c*t)*(sin(x)**2 + cos(y)**2)

    h1 = -sin(x)**2 - cos(y)**2
    h2 = -sin(x)**2 + sin(y)**2 - 1

    # there is still some non-deterministic behavior in integrate
    # or trigsimp which permits one of the following
    assert integrate(c*(P2 - P1), t) in [
        c*(-A*(-h1)*log(c*t)/c + A*t*exp(-z)),
        c*(-A*(-h2)*log(c*t)/c + A*t*exp(-z)),
        c*(A*h1*log(c*t)/c + A*t*exp(-z)),
        c*(A*h2*log(c*t)/c + A*t*exp(-z)),
        (A*c*t - A*(-h1)*log(t)*exp(z))*exp(-z),
        (A*c*t - A*(-h2)*log(t)*exp(z))*exp(-z),
    ]


def test_sympyissue_4892b():
    # Issues relating to issue sympy/sympy#4596 are making the actual result of this hard
    # to test.  The answer should be something like
    #
    # (-sin(y) + sqrt(-72 + 48*cos(y) - 8*cos(y)**2)/2)*log(x + sqrt(-72 +
    # 48*cos(y) - 8*cos(y)**2)/(2*(3 - cos(y)))) + (-sin(y) - sqrt(-72 +
    # 48*cos(y) - 8*cos(y)**2)/2)*log(x - sqrt(-72 + 48*cos(y) -
    # 8*cos(y)**2)/(2*(3 - cos(y)))) + x**2*sin(y)/2 + 2*x*cos(y)

    expr = (sin(y)*x**3 + 2*cos(y)*x**2 + 12)/(x**2 + 2)
    assert trigsimp(factor(integrate(expr, x).diff(x) - expr)) == 0


def test_sympyissue_5178():
    assert integrate(sin(x)*f(y, z), (x, 0, pi), (y, 0, pi), (z, 0, pi)) == \
        2*Integral(f(y, z), (y, 0, pi), (z, 0, pi))


def test_integrate_series():
    f = sin(x).series(x, 0, 10)
    g = x**2/2 - x**4/24 + x**6/720 - x**8/40320 + \
        x**10/3628800 + Integral(O(x**10), x)

    assert integrate(f, x) == g

    assert integrate(O(x**5), x) == Integral(O(x**5), x)


def test_atom_bug():
    assert heurisch(meijerg([], [], [1], [], x), x) is None


def test_limit_bug():
    z = Symbol('z', nonzero=True)
    assert integrate(sin(x*y*z), (x, 0, pi), (y, 0, pi)) == \
        (log(z**2) + 2*EulerGamma + 2*log(pi))/(2*z) - \
        (-log(pi*z) + log(pi**2*z**2)/2 + Ci(pi**2*z))/z + log(pi)/z


def test_sympyissue_4703():
    g = Function('g')
    assert integrate(exp(x)*g(x), x).has(Integral)


def test_sympyissue_1888():
    f = Function('f')
    assert integrate(f(x).diff(x)**2, x).has(Integral)

# The following tests work using meijerint.


def test_sympyissue_3558():
    assert integrate(cos(x*y), (x, -pi/2, pi/2), (y, 0, pi)) == 2*Si(pi**2/2)


def test_sympyissue_4422():
    assert integrate(1/sqrt(16 + 4*x**2), x) == asinh(x/2) / 2


def test_sympyissue_4493():
    assert simplify(integrate(x*sqrt(1 + 2*x), x)) == \
        sqrt(2*x + 1)*(6*x**2 + x - 1)/15


def test_sympyissue_4737():
    assert integrate(sin(x)/x, (x, -oo, oo)) == pi
    assert integrate(sin(x)/x, (x, 0, oo)) == pi/2
    assert integrate(sin(x)/x, x) == Si(x)


def test_sympyissue_4992():
    # Note: psi in _check_antecedents becomes NaN.
    a = Symbol('a', positive=True)
    assert simplify(expand_func(integrate(exp(-x)*log(x)*x**a, (x, 0, oo)))) == \
        (a*polygamma(0, a) + 1)*gamma(a)


def test_sympyissue_4487():
    assert simplify(integrate(exp(-x)*x**y, x)) == lowergamma(y + 1, x)


@pytest.mark.xfail
def test_sympyissue_4215():
    assert integrate(1/x**2, (x, -1, 1)) == oo


def test_sympyissue_4400():
    n = Symbol('n', integer=True, positive=True)
    assert integrate((x**n)*log(x), x) == \
        n*x*x**n*log(x)/(n**2 + 2*n + 1) + x*x**n*log(x)/(n**2 + 2*n + 1) - \
        x*x**n/(n**2 + 2*n + 1)


def test_sympyissue_6253():
    # Note: this used to raise NotImplementedError
    # Note: psi in _check_antecedents becomes NaN.
    assert integrate((sqrt(1 - x) + sqrt(1 + x))**2/x, x, meijerg=True) == \
        Integral((sqrt(-x + 1) + sqrt(x + 1))**2/x, x)


def test_sympyissue_4153():
    assert integrate(1/(1 + x + y + z), (x, 0, 1), (y, 0, 1), (z, 0, 1)) in [
        -12*log(3) - 3*log(6)/2 + 3*log(8)/2 + 5*log(2) + 7*log(4),
        6*log(2) + 8*log(4) - 27*log(3)/2, 22*log(2) - 27*log(3)/2,
        -12*log(3) - 3*log(6)/2 + 47*log(2)/2]


def test_sympyissue_4326():
    # It doesn't matter if we can do the integral.  Just make sure the result
    # doesn't contain nan.  This is really a test against _eval_interval.
    assert not integrate(((h*(x - R + b))/b)*sqrt(R**2 - x**2), (x, R - b, R)).has(nan)


def test_powers():
    assert integrate(2**x + 3**x, x) == 2**x/log(2) + 3**x/log(3)


def test_risch_option():
    # risch=True only allowed on indefinite integrals
    pytest.raises(ValueError, lambda: integrate(1/log(x), (x, 0, oo), risch=True))
    assert integrate(exp(-x**2), x, risch=True) == NonElementaryIntegral(exp(-x**2), x)
    assert integrate(log(1/x)*y, x, y, risch=True) == y**2*(x*log(1/x)/2 + x/2)
    assert integrate(erf(x), x, risch=True) == Integral(erf(x), x)
    # TODO: How to test risch=False?


def test_sympyissue_6828():
    f = 1/(1.08*x**2 - 4.3)
    g = integrate(f, x).diff(x)
    assert verify_numerically(f, g, tol=1e-12)


@pytest.mark.xfail
def test_integrate_Piecewise_rational_over_reals():
    f = Piecewise(
        (0,                                              t - 478.515625*pi < 0),
        (13.2075145209219*pi/(0.000871222*t + 0.995)**2, t - 478.515625*pi >= 0))

    assert integrate(f, (t, 0, oo)) != 0  # ~20664.5


def test_sympyissue_4803():
    x_max = Symbol('x_max')
    assert integrate(y/pi*exp(-(x_max - x)/cos(a)), x) == \
        y*exp((x - x_max)/cos(a))*cos(a)/pi


def test_sympyissue_4234():
    assert integrate(1/sqrt(1 + tan(x)**2)) == tan(x) / sqrt(1 + tan(x)**2)


def test_sympyissue_4492():
    assert simplify(integrate(x**2 * sqrt(5 - x**2), x)) == Piecewise(
        (I*(2*x**5 - 15*x**3 + 25*x - 25*sqrt(x**2 - 5)*acosh(sqrt(5)*x/5)) /
            (8*sqrt(x**2 - 5)), 1 < abs(x**2)/5),
        ((-2*x**5 + 15*x**3 - 25*x + 25*sqrt(-x**2 + 5)*asin(sqrt(5)*x/5)) /
            (8*sqrt(-x**2 + 5)), True))


def test_sympyissue_2708():
    # This test needs to use an integration function that can
    # not be evaluated in closed form.  Update as needed.
    f = 1/(a + z + log(z))
    integral_f = NonElementaryIntegral(f, (z, 2, 3))
    assert Integral(f, (z, 2, 3)).doit() == integral_f
    assert integrate(f + exp(z), (z, 2, 3)) == integral_f - exp(2) + exp(3)

    assert integrate(2*f + exp(z), (z, 2, 3)) == 2*integral_f - exp(2) + exp(3)
    assert (integrate(exp(1.2*n*s*z*(-t + z)/t), (z, 0, x)) ==
            1.0*NonElementaryIntegral(exp(-1.2*n*s*z)*exp(1.2*n*s*z**2/t),
                                      (z, 0, x)))


def test_sympyissue_8368():
    assert integrate(exp(-s*x)*cosh(x), (x, 0, oo)) == \
        Piecewise((pi*Piecewise((-s/(pi*(-s**2 + 1)), abs(s**2) < 1),
                                (1/(pi*s*(1 - 1/s**2)), abs(s**-2) < 1), (meijerg(((Rational(1, 2),), (0, 0)),
                                                                                  ((0, Rational(1, 2)), (0,)), polar_lift(s)**2), True)),
                   And(abs(periodic_argument(polar_lift(s)**2, oo)) < pi, Ne(s**2, 1),
                       cos(abs(periodic_argument(polar_lift(s)**2, oo))/2)*sqrt(abs(s**2)) -
                       1 > 0)), (Integral(exp(-s*x)*cosh(x), (x, 0, oo)), True))
    assert integrate(exp(-s*x)*sinh(x), (x, 0, oo)) == \
        Piecewise((pi*Piecewise((2/(pi*(2*s**2 - 2)), abs(s**2) < 1),
                                (-2/(pi*s**2*(-2 + 2/s**2)), abs(s**-2) < 1),
                                (meijerg(((0,), (Rational(-1, 2), Rational(1, 2))),
                                         ((0, Rational(1, 2)), (Rational(-1, 2),)),
                                         polar_lift(s)**2), True)),
                   And(abs(periodic_argument(polar_lift(s)**2, oo)) < pi, Ne(s**2, 1),
                       cos(abs(periodic_argument(polar_lift(s)**2, oo))/2)*sqrt(abs(s**2)) - 1 > 0)),
                  (Integral(E**(-s*x)*sinh(x), (x, 0, oo)), True))


def test_sympyissue_8901():
    assert integrate(sinh(1.0*x)) == 1.0*cosh(1.0*x)
    assert integrate(tanh(1.0*x)) == 1.0*x - 1.0*log(tanh(1.0*x) + 1)
    assert integrate(tanh(x)) == x - log(tanh(x) + 1)


def test_sympyissue_8945():
    assert integrate(sin(x)**3/x, (x, 0, 1)) == -Si(3)/4 + 3*Si(1)/4
    assert integrate(sin(x)**3/x, (x, 0, oo)) == pi/4
    assert integrate(cos(x)**2/x**2, x) == -Si(2*x) - cos(2*x)/(2*x) - 1/(2*x)


def test_diofantissue_303():
    assert integrate((cos(x)/x)**2, (x, pi, 2*pi)) == Si(2*pi) - Si(4*pi) + 1/pi/2


@pytest.mark.slow
def test_sympyissue_7130():
    integrand = (cos(pi*i*x/L)**2 / (a + b*x)).rewrite(exp)
    assert x not in integrate(integrand, (x, 0, L)).free_symbols


def test_sympyissue_4950():
    assert integrate((-60*exp(x) - 19.2*exp(4*x))*exp(4*x), x) ==\
        -2.4*exp(8*x) - 12.0*exp(5*x)


def test_sympyissue_4968():
    assert integrate(sin(log(x**2))) == x*sin(2*log(x))/5 - 2*x*cos(2*log(x))/5


def test_sympyissue_7098():
    assert integrate(1/sqrt(x) * 1/sqrt(1 - x), (x, 0, 1)) == pi


def test_sympyissue_4187():
    assert integrate(log(x)*exp(-x), (x, 0, oo)) == -EulerGamma


def test_sympyissue_10567():
    vt = Matrix([a*t, b, c])
    assert integrate(vt, t) == Integral(vt, t).doit()
    assert integrate(vt, t) == Matrix([[a*t**2/2], [b*t], [c*t]])


def test_sympyissue_4231():
    e = (1 + 2*x + sqrt(x + log(x))*(1 + 3*x) +
         x**2)/(x*(x + sqrt(x + log(x)))*sqrt(x + log(x)))
    assert integrate(e, x) == 2*log(x + sqrt(x + log(x))) + 2*sqrt(x + log(x))


def test_sympyissue_11045():
    e = 1/(x*sqrt(x**2 - 1))
    assert integrate(e, (x, 1, 2)) == pi/3


def test_definite_integrals_abs():
    # issue sympy/sympy#8430
    assert integrate(abs(x), (x, 0, 1)) == Rational(1, 2)
    # issue sympy/sympy#7165
    r = Symbol('r', real=True)
    assert (integrate(abs(x - r**2), (x, 0, 2)) ==
            r**2*Max(0, Min(2, r**2)) + r**2*Min(2, r**2) -
            2*r**2 - Max(0, Min(2, r**2))**2/2 -
            Min(2, r**2)**2/2 + 2)
    # issue sympy/sympy#8733
    assert integrate(abs(x + 1), (x, 0, 1)) == Rational(3, 2)

    e = x*abs(x**2 - 9)
    assert integrate(e, (x, -2, 2)) == 0
    assert integrate(e, (x, -1, 2)) == Rational(39, 4)
    assert integrate(e, (x, -2, 7)) == Rational(1625, 4)
    assert integrate(e, (x, -3, 11)) == 3136
    assert integrate(e, (x, -17, -2)) == Rational(-78425, 4)
    assert integrate(e, (x, -17, 20)) == Rational(74481, 4)


def test_sympyissue_12081():
    assert integrate(x**(-Rational(3, 2))*exp(-x), (x, 0, oo)) == oo


def test_sympyissue_7163():
    integrate((sign(x - 1) - sign(x - 2))*cos(x), x)  # not raises


def test_sympyissue_12221():
    e = sqrt(1 - x)/x
    r = 2*I*(-sqrt(2) - asin(sqrt(3)/3) + asin(sqrt(5)/5) + 2)
    assert integrate(e, (x, 3, 5)).simplify() == r


def test_sympyissue_12582():
    assert integrate(abs(x**2 - 3*x), (x, -15, 15)) == 2259


def test_sympyissue_13312():
    assert integrate(exp(-a*t), (t, b, oo)) == Piecewise((-b + oo, Eq(a, 0)),
                                                         (-exp(-oo*sign(a))/a + exp(-a*b)/a, True))


def test_sympyissue_10680():
    integrate(x**log(x**log(x**log(x))), x)  # not raises


def test_sympyissue_13501():
    a = Symbol('a', real=True)
    assert integrate(1/(1 + a**2*x**2), x) == atan(a*x)/a


def test_diofantissue_447():
    assert integrate(1/(2*sin(x) + cos(x)),
                     x) == (sqrt(5)*log(tan(x/2) - 2 + sqrt(5))/5 -
                            sqrt(5)*log(tan(x/2) - sqrt(5) - 2)/5)


def test_sympyissue_14782():
    assert integrate(sqrt(-x**2 + 1)*(-x**2 + x), (x, -1, 1)) != 0


def test_sympyissue_4511():
    assert simplify(integrate(cos(x)**2/(1 - sin(x)), x)) in [x - cos(x),
                                                              +1 - cos(x) + x,
                                                              -1 - cos(x) + x,
                                                              -2/(tan(x/2)**2 + 1) + x]


def test_sympyissue_4551():
    assert (integrate(1/(x*sqrt(1 - x**2)), x) ==
            Piecewise((-acosh(1/x), abs(x**-2) > 1), (I*asin(1/x), True)))


@pytest.mark.xfail
def test_sympyissue_4212():
    assert not isinstance(integrate(sign(x), x), Integral)


@pytest.mark.xfail
def test_sympyissue_4491():
    assert not isinstance(integrate(x*sqrt(x**2 + 2*x + 4), x), Integral)


@pytest.mark.xfail
def test_sympyissue_4514():
    assert not isinstance(integrate(sin(2*x)/sin(x)), Integral)


def test_sympyissue_15810():
    assert integrate(1/(2**(2*x/3) + 1), (x, 0, oo)) == Rational(3, 2)


def test_diofantissue_149():
    a = Symbol('a', positive=True)
    expr = (2 - x)**a*sin(a/x)
    res = sqrt(pi)*a*meijerg(((), (a/2 + 1/2, a/2 + 1)),
                             ((0, 0, Rational(1, 2)), (Rational(-1, 2),)),
                             a**2/16)*gamma(a + 1)/4
    assert integrate(expr, (x, 0, 2)) == res


def test_sympyissue_7337():
    assert integrate(x*sqrt(2*x + 3), (x, -1, 1)) == Rational(2, 5)


def test_sympyissue_11877():
    assert integrate(log(Rational(1, 2) - x),
                     (x, 0, Rational(1, 2))) == -Rational(1, 2) - log(2)/2


def test_sympyissue_17841():
    e = 1/(x**2 + x + I)
    assert integrate(e.diff(x), x) == e


def test_sympyissue_18384():
    e = abs(sin(x)*cos(x))
    assert integrate(e, (x, pi, 2*pi)) == 1
    assert integrate(e, (x, 0, pi/2)) == Rational(1, 2)
    assert integrate(e, (x, pi/2, pi)) == Rational(1, 2)
    assert integrate(e, (x, pi, 3*pi/2)) == Rational(1, 2)
    assert integrate(e, (x, 3*pi/2, 2*pi)) == Rational(1, 2)



"""Generic Rules for Diofant.

This file assumes knowledge of Basic and little else.
"""

from strategies.dispatch import dispatch

from ..utilities.iterables import sift
from .basic import Atom, Basic


__all__ = ('arguments', 'operator', 'term', 'rm_id',
           'glom', 'flatten', 'unpack', 'sort')


@dispatch(Basic)
def arguments(o):
    return o.args


@arguments.register((int, Atom))
def arguments_atomic(o):
    return ()


@dispatch(Basic)
def operator(o):
    return o.func


@operator.register((int, Atom))
def operator_atomic(o):
    return o


@dispatch(type, (tuple, list))
def term(op, args):
    return op(*args)


@term.register((int, Atom), (tuple, list))
def term_atomic(op, args):
    return op


# Functions that create rules


def rm_id(isid):
    """Create a rule to remove identities

    isid - fn :: x -> Bool  --- whether or not this element is an identity

    >>> remove_zeros = rm_id(lambda x: x == 0)
    >>> remove_zeros(Basic(1, 0, 2))
    Basic(1, 2)
    >>> remove_zeros(Basic(0, 0))  # If only identites then we keep one
    Basic(0)

    See Also
    ========

    unpack

    """
    def ident_remove(expr):
        """Remove identities."""
        ids = list(map(isid, arguments(expr)))
        if sum(ids) == 0:           # No identities. Common case
            return expr
        elif sum(ids) != len(ids):  # there is at least one non-identity
            return term(operator(expr),
                        [arg for arg, x in zip(arguments(expr), ids) if not x])
        else:
            return term(operator(expr), [arguments(expr)[0]])

    return ident_remove


def glom(key, count, combine):
    """Create a rule to conglomerate identical args.

    >>> def key(x):
    ...     return x.as_coeff_Mul()[1]
    >>> def count(x):
    ...     return x.as_coeff_Mul()[0]
    >>> def combine(cnt, arg):
    ...     return cnt * arg
    >>> rl = glom(key, count, combine)

    >>> rl(Add(x, -x, 3*x, 2, 3, evaluate=False))
    3*x + 5

    Wait, how are key, count and combine supposed to work?

    >>> key(2*x)
    x
    >>> count(2*x)
    2
    >>> combine(2, x)
    2*x

    """
    def conglomerate(expr):
        """Conglomerate together identical args x + x -> 2x."""
        groups = sift(arguments(expr), key)
        counts = {k: sum(map(count, args)) for k, args in groups.items()}
        newargs = [combine(cnt, mat) for mat, cnt in counts.items()]
        if set(newargs) != set(arguments(expr)):
            return term(operator(expr), newargs)
        else:
            return expr

    return conglomerate


def sort(key):
    """Create a rule to sort by a key function.

    >>> sort_rl = sort(str)
    >>> sort_rl(Basic(3, 1, 2))
    Basic(1, 2, 3)

    """

    def sort_rl(expr):
        return term(operator(expr), sorted(arguments(expr), key=key))
    return sort_rl


# Functions that are rules


def unpack(expr):
    """Rule to unpack singleton args.

    >>> unpack(Basic(2))
    2

    """
    if len(arguments(expr)) == 1:
        return arguments(expr)[0]
    else:
        return expr


def flatten(expr):
    """Flatten T(a, b, T(c, d), T2(e)) to T(a, b, c, d, T2(e))."""
    cls = operator(expr)
    args = []
    for arg in arguments(expr):
        if operator(arg) == cls:
            args.extend(arguments(arg))
        else:
            args.append(arg)
    return term(cls, args)



"""Tools to assist importing optional external modules."""

import sys
import warnings
from distutils.version import LooseVersion

from .. import DIOFANT_DEBUG


def import_module(module, min_module_version=None, min_python_version=None,
                  warn_not_installed=False, warn_old_version=True,
                  module_version_attr='__version__',
                  module_version_attr_call_args=None,
                  __import__kwargs={}, catch=()):
    """
    Import and return a module if it is installed.

    If the module is not installed, it returns None.

    A minimum version for the module can be given as the keyword argument
    min_module_version.  This should be comparable against the module version.
    By default, module.__version__ is used to get the module version.  To
    override this, set the module_version_attr keyword argument.  If the
    attribute of the module to get the version should be called (e.g.,
    module.version()), then set module_version_attr_call_args to the args such
    that module.module_version_attr(*module_version_attr_call_args) returns the
    module's version.

    If the module version is less than min_module_version using the Python <
    comparison, None will be returned, even if the module is installed. You can
    use this to keep from importing an incompatible older version of a module.

    You can also specify a minimum Python version by using the
    min_python_version keyword argument.  This should be comparable against
    sys.version_info.

    If the keyword argument warn_not_installed is set to True (or
    DIOFANT_DEBUG is True), the function will emit a UserWarning when
    the module is not installed.

    By default, or if the keyword argument warn_old_version is set to True (or
    DIOFANT_DEBUG is True), the function will emit a UserWarning when
    the library is installed, but cannot be imported because of the
    min_module_version or min_python_version options.

    This function uses __import__() to import the module.  To pass additional
    options to __import__(), use the __import__kwargs keyword argument.  For
    example, to import a submodule A.B, you must pass a nonempty fromlist option
    to __import__.

    This catches ImportError to determine if the module is not installed.  To
    catch additional errors, pass them as a tuple to the catch keyword
    argument.

    Examples
    ========

    >>> numpy = import_module('numpy')

    >>> numpy = import_module('numpy', min_python_version=(2, 7),
    ...                       warn_old_version=False)

    numpy.__version__ is a string

    >>> numpy = import_module('numpy', min_module_version='1.5',
    ...                       warn_old_version=False)

    gmpy does not have __version__, but it does have gmpy.version()

    >>> gmpy = import_module('gmpy2', min_module_version='2.0.0',
    ...                      module_version_attr='version',
    ...                      module_version_attr_call_args=(),
    ...                      warn_old_version=False)

    To import a submodule, you must pass a nonempty fromlist to
    __import__().  The values do not matter.

    >>> p3 = import_module('mpl_toolkits.mplot3d',
    ...                    __import__kwargs={'fromlist': ['something']})

    matplotlib.pyplot can raise RuntimeError when the display cannot be opened

    >>> matplotlib = import_module('matplotlib',
    ...                            __import__kwargs={'fromlist': ['pyplot']},
    ...                            catch=(RuntimeError,))

    See Also
    ========

    __import__

    """
    warn_old_version = True if DIOFANT_DEBUG else warn_old_version
    warn_not_installed = True if DIOFANT_DEBUG else warn_not_installed

    # Check Python first so we don't waste time importing a module we can't use
    if min_python_version:
        if sys.version_info < min_python_version:
            min_python_version = '.'.join(map(str, min_python_version))
            if warn_old_version:
                warnings.warn('Python version is too old to use %s '
                              '(%s or newer required)' % (module,
                                                          min_python_version),
                              UserWarning)
            return

    try:
        mod = __import__(module, **__import__kwargs)
    except ImportError:
        if warn_not_installed:
            warnings.warn(f'{module} module is not installed', UserWarning)
        return

    if min_module_version:
        modversion = getattr(mod, module_version_attr)
        if module_version_attr_call_args is not None:
            modversion = modversion(*module_version_attr_call_args)
        if LooseVersion(modversion) < LooseVersion(min_module_version):
            if warn_old_version:
                warnings.warn('%s version is too old to use '
                              '(%s or newer required)' % (module,
                                                          min_module_version),
                              UserWarning)
            return

    return mod



from diofant import (Derivative, Float, Integer, Integral, Order, Product,
                     Rational, Rel, Sum, oo, sin)
from diofant.abc import x, y
from diofant.printing.precedence import PRECEDENCE, precedence


__all__ = ()


def test_Add():
    assert precedence(x + y) == PRECEDENCE['Add']
    assert precedence(x*y + 1) == PRECEDENCE['Add']


def test_Function():
    assert precedence(sin(x)) == PRECEDENCE['Func']


def test_Derivative():
    assert precedence(Derivative(x, y)) == PRECEDENCE['Atom']


def test_Integral():
    assert precedence(Integral(x, y)) == PRECEDENCE['Atom']


def test_Mul():
    assert precedence(x*y) == PRECEDENCE['Mul']
    assert precedence(-x*y) == PRECEDENCE['Add']


def test_Number():
    assert precedence(Integer(0)) == PRECEDENCE['Atom']
    assert precedence(Integer(1)) == PRECEDENCE['Atom']
    assert precedence(Integer(-1)) == PRECEDENCE['Add']
    assert precedence(Integer(10)) == PRECEDENCE['Atom']
    assert precedence(Rational(5, 2)) == PRECEDENCE['Mul']
    assert precedence(Rational(-5, 2)) == PRECEDENCE['Add']
    assert precedence(Float(5)) == PRECEDENCE['Atom']
    assert precedence(Float(-5)) == PRECEDENCE['Add']
    assert precedence(oo) == PRECEDENCE['Atom']
    assert precedence(-oo) == PRECEDENCE['Add']


def test_Order():
    assert precedence(Order(x)) == PRECEDENCE['Atom']


def test_Pow():
    assert precedence(x**y) == PRECEDENCE['Pow']
    assert precedence(-x**y) == PRECEDENCE['Add']
    assert precedence(x**-y) == PRECEDENCE['Pow']


def test_Product():
    assert precedence(Product(x, (x, y, y + 1))) == PRECEDENCE['Atom']


def test_Relational():
    assert precedence(Rel(x + y, y, '<')) == PRECEDENCE['Relational']


def test_Sum():
    assert precedence(Sum(x, (x, y, y + 1))) == PRECEDENCE['Atom']


def test_Symbol():
    assert precedence(x) == PRECEDENCE['Atom']


def test_And_Or():
    # precendence relations between logical operators, ...
    assert precedence(x & y) > precedence(x | y)
    assert precedence(~y) > precedence(x & y)
    # ... and with other operators (cfr. other programming languages)
    assert precedence(x + y) > precedence(x | y)
    assert precedence(x + y) > precedence(x & y)
    assert precedence(x*y) > precedence(x | y)
    assert precedence(x*y) > precedence(x & y)
    assert precedence(~y) > precedence(x*y)
    assert precedence(~y) > precedence(x - y)
    # double checks
    assert precedence(x & y) == PRECEDENCE['And']
    assert precedence(x | y) == PRECEDENCE['Or']
    assert precedence(~y) == PRECEDENCE['Not']



"""Diofant is a Python library for symbolic mathematics."""

import os
DIOFANT_DEBUG = os.getenv('DIOFANT_DEBUG', 'False') != 'False'
del os

import pkg_resources
__version__ = pkg_resources.get_distribution(__name__).version
del pkg_resources

from .core import *
from .logic import *
from .polys import *
from .domains import *
from .series import *
from .functions import *
from .ntheory import *
from .concrete import *
from .simplify import *
from .sets import *
from .solvers import *
from .matrices import *
from .geometry import *
from .utilities import *
from .integrals import *
from .tensor import *
from .calculus import *
from .combinatorics import *
from .plotting import *
from .printing import *
from .interactive import *



import sys

import pytest

import diofant


collect_ignore = ['setup.py']
try:
    import matplotlib
    matplotlib.rc('figure', max_open_warning=0)
    del matplotlib
except ImportError:
    collect_ignore_glob = ['diofant/plotting/*.py']


def pytest_report_header(config):
    return f"""\nDiofant version: {diofant.__version__}
cache: {diofant.core.cache.USE_CACHE}
ground types: {diofant.core.compatibility.GROUND_TYPES}\n"""


def pytest_configure(config):
    config.addinivalue_line('markers', 'slow: marks tests as slow')


@pytest.fixture(autouse=True, scope='module')
def file_clear_cache():
    diofant.core.cache.clear_cache()


@pytest.fixture(autouse=True, scope='session')
def set_displayhook():
    sys.__displayhook__ = sys.displayhook  # https://bugs.python.org/26092


@pytest.fixture(autouse=True, scope='session')
def enable_mpl_agg_backend():
    try:
        import matplotlib
        matplotlib.use('Agg')
    except ImportError:
        pass


@pytest.fixture(autouse=True)
def add_np(doctest_namespace):
    for sym in (diofant.symbols('a:d t x:z') +
                diofant.symbols('k m n', integer=True) +
                diofant.symbols('f:h', cls=diofant.Function)):
        doctest_namespace[str(sym)] = sym
    for name in dir(diofant):
        doctest_namespace[name] = getattr(diofant, name)



"""Advanced tools for dense recursive polynomials in ``K[x]`` or ``K[X]``."""

from .densearith import (dmp_add, dmp_add_term, dmp_div, dmp_exquo_ground,
                         dmp_mul, dmp_mul_ground, dmp_neg, dmp_sub, dup_add,
                         dup_mul)
from .densebasic import (dmp_convert, dmp_degree_in, dmp_from_dict, dmp_ground,
                         dmp_ground_LC, dmp_LC, dmp_strip, dmp_TC, dmp_to_dict,
                         dmp_zero, dmp_zero_p)
from .polyerrors import DomainError


def dmp_diff_in(f, m, j, u, K):
    """
    ``m``-th order derivative in ``x_j`` of a polynomial in ``K[X]``.

    Examples
    ========

    >>> R, x, y = ring('x y', ZZ)

    >>> f = x*y**2 + 2*x*y + 3*x + 2*y**2 + 3*y + 1

    >>> f.diff()
    y**2 + 2*y + 3
    >>> f.diff(y)
    2*x*y + 2*x + 4*y + 3

    """
    ring = K.poly_ring(*[f'_{i}' for i in range(u + 1)])
    f = ring.from_dense(f)
    return ring.to_dense(f.diff(x=j, m=m))


def dmp_eval_in(f, a, j, u, K):
    """
    Evaluate a polynomial at ``x_j = a`` in ``K[X]`` using the Horner scheme.

    Examples
    ========

    >>> R, x, y = ring('x y', ZZ)

    >>> f = 2*x*y + 3*x + y + 2

    >>> R.dmp_eval_in(f, 2, 0)
    5*y + 8
    >>> R.dmp_eval_in(f, 2, 1)
    7*x + 4

    """
    if j < 0 or j > u:
        raise IndexError(f'0 <= j <= {u} expected, got {j}')

    if not j:
        if not a:
            return dmp_TC(f, K)

        result, v = dmp_LC(f, K), u - 1

        if u:
            for coeff in f[1:]:
                result = dmp_mul_ground(result, a, v, K)
                result = dmp_add(result, coeff, v, K)
        else:
            for coeff in f[1:]:
                result *= a
                result += coeff

        return result

    def eval_in(g, a, v, i, j, K):
        if i == j:
            return dmp_eval_in(g, a, 0, v, K)

        v, i = v - 1, i + 1

        return dmp_strip([eval_in(c, a, v, i, j, K) for c in g], v)

    return eval_in(f, a, u, 0, j, K)


def dmp_eval_tail(f, A, u, K):
    """
    Evaluate a polynomial at ``x_j = a_j, ...`` in ``K[X]``.

    Examples
    ========

    >>> R, x, y = ring('x y', ZZ)

    >>> f = 2*x*y + 3*x + y + 2

    >>> R.dmp_eval_tail(f, [2])
    7*x + 4
    >>> R.dmp_eval_tail(f, [2, 2])
    18

    """
    if not A:
        return f

    if dmp_zero_p(f, u):
        return dmp_zero(u - len(A))

    def eval_tail(g, i, A, u, K):
        if i == u:
            return dmp_eval_in(g, A[-1], 0, 0, K)
        else:
            h = [eval_tail(c, i + 1, A, u, K) for c in g]

            if i < u - len(A) + 1:
                return h
            else:
                return dmp_eval_in(h, A[-u + i - 1], 0, 0, K)

    e = eval_tail(f, 0, A, u, K)

    if u == len(A) - 1:
        return e
    else:
        return dmp_strip(e, u - len(A))


def dmp_diff_eval_in(f, m, a, j, u, K):
    """
    Differentiate and evaluate a polynomial in ``x_j`` at ``a`` in ``K[X]``.

    Examples
    ========

    >>> R, x, y = ring('x y', ZZ)

    >>> f = x*y**2 + 2*x*y + 3*x + 2*y**2 + 3*y + 1

    >>> R.dmp_diff_eval_in(f, 1, 2, 0)
    y**2 + 2*y + 3
    >>> R.dmp_diff_eval_in(f, 1, 2, 1)
    6*x + 11

    """
    if j > u:
        raise IndexError(f'-{u} <= j < {u} expected, got {j}')
    if not j:
        return dmp_eval_in(dmp_diff_in(f, m, 0, u, K), a, 0, u, K)

    def diff_eval(g, m, a, v, i, j, K):
        if i == j:
            return dmp_eval_in(dmp_diff_in(g, m, 0, v, K), a, 0, v, K)

        v, i = v - 1, i + 1

        return dmp_strip([diff_eval(c, m, a, v, i, j, K) for c in g], v)

    return diff_eval(f, m, a, u, 0, j, K)


def dup_trunc(f, p, K):
    """
    Reduce a ``K[x]`` polynomial modulo a constant ``p`` in ``K``.

    Examples
    ========

    >>> R, x = ring('x', ZZ)

    >>> R.dmp_ground_trunc(2*x**3 + 3*x**2 + 5*x + 7, ZZ(3))
    -x**3 - x + 1

    """
    from ..ntheory.modular import symmetric_residue

    if K.is_IntegerRing:
        g = []

        for c in f:
            c = c % p
            c = symmetric_residue(c, p)
            g.append(c)
    else:
        g = [c % p for c in f]

    return dmp_strip(g, 0)


def dmp_ground_trunc(f, p, u, K):
    """
    Reduce a ``K[X]`` polynomial modulo a constant ``p`` in ``K``.

    Examples
    ========

    >>> R, x, y = ring('x y', ZZ)

    >>> f = 3*x**2*y + 8*x**2 + 5*x*y + 6*x + 2*y + 3

    >>> R.dmp_ground_trunc(f, ZZ(3))
    -x**2 - x*y - y

    """
    if not u:
        return dup_trunc(f, p, K)

    v = u - 1

    return dmp_strip([dmp_ground_trunc(c, p, v, K) for c in f], u)


def dmp_ground_monic(f, u, K):
    """
    Divide all coefficients by ``LC(f)`` in ``K[X]``.

    Examples
    ========

    >>> R, x, y = ring('x y', ZZ)
    >>> f = 3*x**2*y + 6*x**2 + 3*x*y + 9*y + 3

    >>> R.dmp_ground_monic(f)
    x**2*y + 2*x**2 + x*y + 3*y + 1

    >>> R, x, y = ring('x y', QQ)
    >>> f = 3*x**2*y + 8*x**2 + 5*x*y + 6*x + 2*y + 3

    >>> R.dmp_ground_monic(f)
    x**2*y + 8/3*x**2 + 5/3*x*y + 2*x + 2/3*y + 1

    """
    if dmp_zero_p(f, u):
        return f

    lc = dmp_ground_LC(f, u, K)

    if lc == K.one:
        return f
    else:
        return dmp_exquo_ground(f, lc, u, K)


def dmp_ground_content(f, u, K):
    """
    Compute the GCD of coefficients of ``f`` in ``K[X]``.

    Examples
    ========

    >>> R, x, y = ring('x y', ZZ)
    >>> f = 2*x*y + 6*x + 4*y + 12

    >>> f.content()
    2

    >>> R, x, y = ring('x y', QQ)
    >>> f = 2*x*y + 6*x + 4*y + 12

    >>> f.content()
    2

    """
    ring = K.poly_ring(*[f'_{i}' for i in range(u + 1)])
    f = ring.from_dense(f)
    return f.content()


def dmp_ground_primitive(f, u, K):
    """
    Compute content and the primitive form of ``f`` in ``K[X]``.

    Examples
    ========

    >>> R, x, y = ring('x y', ZZ)
    >>> f = 2*x*y + 6*x + 4*y + 12

    >>> f.primitive()
    (2, x*y + 3*x + 2*y + 6)

    >>> R, x, y = ring('x y', QQ)
    >>> f = 2*x*y + 6*x + 4*y + 12

    >>> f.primitive()
    (2, x*y + 3*x + 2*y + 6)

    """
    ring = K.poly_ring(*[f'_{i}' for i in range(u + 1)])
    f = ring.from_dense(f)
    cont, p = f.primitive()
    return cont, ring.to_dense(p)


def dup_real_imag(f, K):
    """
    Return bivariate polynomials ``f1`` and ``f2``, such that ``f = f1 + f2*I``.

    Examples
    ========

    >>> R, x, y = ring('x y', ZZ)

    >>> R.dup_real_imag(x**3 + x**2 + x + 1)
    (x**3 + x**2 - 3*x*y**2 + x - y**2 + 1, 3*x**2*y + 2*x*y - y**3 + y)

    >>> R, x, y = ring('x y', QQ.algebraic_field(I))

    >>> R.dup_real_imag(x**2 + I*x - 1)
    (x**2 - y**2 - y - 1, 2*x*y + x)

    """
    if K.is_ComplexAlgebraicField:
        K0 = K.domain
        r1, i1 = dup_real_imag([_.real for _ in f], K0)
        r2, i2 = dup_real_imag([_.imag for _ in f], K0)
        return dmp_add(r1, dmp_neg(i2, 1, K0), 1, K0), dmp_add(r2, i1, 1, K0)
    elif not K.is_IntegerRing and not K.is_RationalField and not K.is_RealAlgebraicField:
        raise DomainError(f'computing real and imaginary parts is not supported over {K}')

    f1 = dmp_zero(1)
    f2 = dmp_zero(1)

    if not f:
        return f1, f2

    g = [[[K.one, K.zero]], [[K.one], []]]
    h = dmp_ground(f[0], 2)

    for c in f[1:]:
        h = dmp_mul(h, g, 2, K)
        h = dmp_add_term(h, dmp_ground(c, 1), 0, 2, K)

    H = dmp_to_dict(h, 0)

    for (k,), h in H.items():
        m = k % 4

        if not m:
            f1 = dmp_add(f1, h, 1, K)
        elif m == 1:
            f2 = dmp_add(f2, h, 1, K)
        elif m == 2:
            f1 = dmp_sub(f1, h, 1, K)
        else:
            f2 = dmp_sub(f2, h, 1, K)

    return f1, f2


def dup_mirror(f, K):
    """
    Evaluate efficiently the composition ``f(-x)`` in ``K[x]``.

    Examples
    ========

    >>> R, x = ring('x', ZZ)

    >>> R.dup_mirror(x**3 + 2*x**2 - 4*x + 2)
    -x**3 + 2*x**2 + 4*x + 2

    """
    f = list(f)

    for i in range(len(f) - 2, -1, -2):
        f[i] = -f[i]

    return f


def dup_scale(f, a, K):
    """
    Evaluate efficiently composition ``f(a*x)`` in ``K[x]``.

    Examples
    ========

    >>> R, x = ring('x', ZZ)

    >>> R.dup_scale(x**2 - 2*x + 1, ZZ(2))
    4*x**2 - 4*x + 1

    """
    f, n, b = list(f), len(f) - 1, a

    for i in range(n - 1, -1, -1):
        f[i], b = b*f[i], b*a

    return f


def dup_shift(f, a, K):
    """
    Evaluate efficiently Taylor shift ``f(x + a)`` in ``K[x]``.

    Examples
    ========

    >>> R, x = ring('x', ZZ)

    >>> R.dup_shift(x**2 - 2*x + 1, ZZ(2))
    x**2 + 2*x + 1

    """
    f, n = list(f), len(f) - 1

    for i in range(n, 0, -1):
        for j in range(i):
            f[j + 1] += a*f[j]

    return f


def dup_transform(f, p, q, K):
    """
    Evaluate functional transformation ``q**n * f(p/q)`` in ``K[x]``.

    Examples
    ========

    >>> R, x = ring('x', ZZ)

    >>> R.dup_transform(x**2 - 2*x + 1, x**2 + 1, x - 1)
    x**4 - 2*x**3 + 5*x**2 - 4*x + 4

    """
    if not f:
        return []

    n = len(f) - 1
    h, Q = [f[0]], [[K.one]]

    for i in range(n):
        Q.append(dup_mul(Q[-1], q, K))

    for c, q in zip(f[1:], Q[1:]):
        h = dup_mul(h, p, K)
        q = dmp_mul_ground(q, c, 0, K)
        h = dup_add(h, q, K)

    return h


def dmp_compose(f, g, u, K):
    """
    Evaluate functional composition ``f(g)`` in ``K[X]``.

    Examples
    ========

    >>> R, x, y = ring('x y', ZZ)

    >>> R.dmp_compose(x*y + 2*x + y, y)
    y**2 + 3*y

    """
    if dmp_zero_p(f, u):
        return f

    h = [f[0]]

    for c in f[1:]:
        h = dmp_mul(h, g, u, K)
        h = dmp_add_term(h, c, 0, u, K)

    return h


def _dup_right_decompose(f, s, K):
    n = len(f) - 1
    lc = dmp_LC(f, K)

    f = dmp_to_dict(f, 0)
    g = {(s,): K.one}

    r = n // s

    for i in range(1, s):
        coeff = K.zero

        for j in range(i):
            if not (n + j - i,) in f:
                continue

            assert (s - j,) in g

            fc, gc = f[(n + j - i,)], g[(s - j,)]
            coeff += (i - r*j)*fc*gc

        g[(s - i,)] = K.quo(coeff, i*r*lc)

    return dmp_from_dict(g, 0, K)


def _dup_left_decompose(f, h, K):
    g, i = {}, 0

    while f:
        q, r = dmp_div(f, h, 0, K)

        if dmp_degree_in(r, 0, 0) > 0:
            return
        else:
            g[(i,)] = dmp_LC(r, K)
            f, i = q, i + 1

    return dmp_from_dict(g, 0, K)


def _dup_decompose(f, K):
    df = len(f) - 1

    for s in range(2, df):
        if df % s != 0:
            continue

        h = _dup_right_decompose(f, s, K)
        g = _dup_left_decompose(f, h, K)

        if g is not None:
            return g, h


def dup_decompose(f, K):
    """
    Compute functional decomposition of ``f`` in ``K[x]``.

    Given a univariate polynomial ``f`` with coefficients in a field of
    characteristic zero, returns list ``[f_1, f_2, ..., f_n]``, where::

              f = f_1 o f_2 o ... f_n = f_1(f_2(... f_n))

    and ``f_2, ..., f_n`` are monic and homogeneous polynomials of at
    least second degree.

    Unlike factorization, complete functional decompositions of
    polynomials are not unique, consider examples:

    1. ``f o g = f(x + b) o (g - b)``
    2. ``x**n o x**m = x**m o x**n``
    3. ``T_n o T_m = T_m o T_n``

    where ``T_n`` and ``T_m`` are Chebyshev polynomials.

    Examples
    ========

    >>> R, x = ring('x', ZZ)

    >>> (x**4 - 2*x**3 + x**2).decompose()
    [x**2, x**2 - x]

    References
    ==========

    * :cite:`Kozen1989decomposition`

    """
    F = []

    while True:
        result = _dup_decompose(f, K)

        if result is not None:
            f, h = result
            F = [h] + F
        else:
            break

    return [f] + F


def dmp_clear_denoms(f, u, K0, K1=None, convert=False):
    """
    Clear denominators, i.e. transform ``K_0`` to ``K_1``.

    Examples
    ========

    >>> R, x, y = ring('x y', QQ)

    >>> f = x/2 + y/3 + 1

    >>> R.dmp_clear_denoms(f, convert=False)
    (6, 3*x + 2*y + 6)
    >>> R.dmp_clear_denoms(f, convert=True)
    (6, 3*x + 2*y + 6)

    """
    if K1 is None:
        if K0.has_assoc_Ring:
            K1 = K0.ring
        else:
            K1 = K0

    def clear_denoms(g, v, K0, K1):
        common = K1.one

        if not v:
            for c in g:
                common = K1.lcm(common, c.denominator)
        else:
            w = v - 1

            for c in g:
                common = K1.lcm(common, clear_denoms(c, w, K0, K1))

        return common

    common = clear_denoms(f, u, K0, K1)
    f = dmp_mul_ground(f, common, u, K0)

    if not convert:
        return common, f
    else:
        return common, dmp_convert(f, u, K0, K1)



"""C code printing tests."""

import pytest

from diofant import (ITE, Abs, Catalan, Dummy, Eq, Equivalent, EulerGamma,
                     GoldenRatio, Idx, IndexedBase, Integer, Lambda, Matrix,
                     MatrixSymbol, Max, Mul, Piecewise, Rational, ccode,
                     ceiling, cos, elliptic_e, exp, gamma, oo, pi, sign, sin,
                     sqrt, symbols)
from diofant.abc import x, y, z
from diofant.printing.ccode import CCodePrinter
from diofant.utilities.lambdify import implemented_function


__all__ = ()


def test_printmethod():
    class Fabs(Abs):
        def _ccode(self, printer):
            return 'fabs(%s)' % printer._print(self.args[0])
    assert ccode(Fabs(x)) == 'fabs(x)'


def test_ccode_sqrt():
    assert ccode(sqrt(x)) == 'sqrt(x)'
    assert ccode(x**0.5) == 'sqrt(x)'
    assert ccode(sqrt(x)) == 'sqrt(x)'


def test_ccode_Pow():
    assert ccode(x**3) == 'pow(x, 3)'
    assert ccode(x**(y**3)) == 'pow(x, pow(y, 3))'
    g = implemented_function('g', Lambda(x, 2*x))
    assert ccode(1/(g(x)*3.5)**(x - y**x)/(x**2 + y)) == \
        'pow(3.5*2*x, -x + pow(y, x))/(pow(x, 2) + y)'
    assert ccode(x**-1.0) == '1.0/x'
    assert ccode(x**Rational(2, 3)) == 'pow(x, 2.0L/3.0L)'
    _cond_cfunc = [(lambda base, exp: exp.is_integer, 'dpowi'),
                   (lambda base, exp: not exp.is_integer, 'pow')]
    assert ccode(x**3, user_functions={'Pow': _cond_cfunc}) == 'dpowi(x, 3)'
    assert ccode(x**3.2, user_functions={'Pow': _cond_cfunc}) == 'pow(x, 3.2)'

    _cond_cfunc2 = [(lambda base, exp: base == 2, lambda base, exp: f'exp2({exp})'),
                    (lambda base, exp: base != 2, 'pow')]
    # Related to sympy/sympy#11353
    assert ccode(2**x, user_functions={'Pow': _cond_cfunc2}) == 'exp2(x)'
    assert ccode(x**2, user_functions={'Pow': _cond_cfunc2}) == 'pow(x, 2)'


def test_ccode_Max():
    # issue sympy/sympy#11926
    assert ccode(Max(x, x*x),
                 user_functions={'Max': 'my_max',
                                 'Pow': 'my_pow'}) == 'my_max(x, my_pow(x, 2))'


def test_ccode_constants_mathh():
    assert ccode(exp(1)) == 'M_E'
    assert ccode(pi) == 'M_PI'
    assert ccode(oo) == 'HUGE_VAL'
    assert ccode(-oo) == '-HUGE_VAL'


def test_ccode_constants_other():
    assert ccode(2*GoldenRatio) == 'double const GoldenRatio = 1.61803398874989;\n2*GoldenRatio'
    assert ccode(
        2*Catalan) == 'double const Catalan = 0.915965594177219;\n2*Catalan'
    assert ccode(2*EulerGamma) == 'double const EulerGamma = 0.577215664901533;\n2*EulerGamma'


def test_ccode_Rational():
    assert ccode(Rational(3, 7)) == '3.0L/7.0L'
    assert ccode(Rational(18, 9)) == '2'
    assert ccode(Rational(3, -7)) == '-3.0L/7.0L'
    assert ccode(Rational(-3, -7)) == '3.0L/7.0L'
    assert ccode(x + Rational(3, 7)) == 'x + 3.0L/7.0L'
    assert ccode(Rational(3, 7)*x) == '(3.0L/7.0L)*x'


def test_ccode_Integer():
    assert ccode(Integer(67)) == '67'
    assert ccode(Integer(-1)) == '-1'


def test_ccode_functions():
    assert ccode(sin(x) ** cos(x)) == 'pow(sin(x), cos(x))'

    assert ccode(elliptic_e(x)) == ('// Not supported in C:\n'
                                    '// elliptic_e\nelliptic_e(x)')

    n = symbols('n', integer=True)
    assert ccode(abs(n)) == '// Not supported in C:\n// Abs\nAbs(n)'
    assert ccode(abs(x)) == 'fabs(x)'

    pytest.raises(TypeError, lambda: ccode(sin(x), assign_to=1))


def test_ccode_inline_function():
    g = implemented_function('g', Lambda(x, 2*x))
    assert ccode(g(x)) == '2*x'
    g = implemented_function('g', Lambda(x, 2*x/Catalan))
    assert ccode(
        g(x)) == 'double const Catalan = %s;\n2*x/Catalan' % Catalan.evalf()
    A = IndexedBase('A')
    i = Idx('i', symbols('n', integer=True))
    g = implemented_function('g', Lambda(x, x*(1 + x)*(2 + x)))
    assert ccode(g(A[i]), assign_to=A[i]) == (
        'for (int i=0; i<n; i++){\n'
        '   A[i] = (A[i] + 1)*(A[i] + 2)*A[i];\n'
        '}'
    )


def test_ccode_exceptions():
    assert ccode(ceiling(x)) == 'ceil(x)'
    assert ccode(abs(x)) == 'fabs(x)'
    assert ccode(gamma(x)) == 'tgamma(x)'


def test_ccode_user_functions():
    x = symbols('x', integer=False)
    n = symbols('n', integer=True)
    custom_functions = {
        'ceiling': 'ceil',
        'Abs': [(lambda x: not x.is_integer, 'fabs'), (lambda x: x.is_integer, 'abs')],
    }
    assert ccode(ceiling(x), user_functions=custom_functions) == 'ceil(x)'
    assert ccode(abs(x), user_functions=custom_functions) == 'fabs(x)'
    assert ccode(abs(n), user_functions=custom_functions) == 'abs(n)'


def test_ccode_boolean():
    assert ccode(x & y) == 'x && y'
    assert ccode(x | y) == 'x || y'
    assert ccode(~x) == '!x'
    assert ccode(x & y & z) == 'x && y && z'
    assert ccode(x | y | z) == 'x || y || z'
    assert ccode((x & y) | z) == 'z || x && y'
    assert ccode((x | y) & z) == 'z && (x || y)'

    assert ccode(x ^ y) == '// Not supported in C:\n// Xor\nXor(x, y)'
    assert ccode(Equivalent(x, y)) == ('// Not supported in C:\n// '
                                       'Equivalent\nEquivalent(x, y)')


def test_ccode_Piecewise():
    expr = Piecewise((x, x < 1), (x**2, True))
    assert ccode(expr) == (
        '((x < 1) ? (\n'
        '   x\n'
        ')\n'
        ': (\n'
        '   pow(x, 2)\n'
        '))')
    assert ccode(expr, assign_to='c') == (
        'if (x < 1) {\n'
        '   c = x;\n'
        '}\n'
        'else {\n'
        '   c = pow(x, 2);\n'
        '}')
    expr = Piecewise((x, x < 1), (x + 1, x < 2), (x**2, True))
    assert ccode(expr) == (
        '((x < 1) ? (\n'
        '   x\n'
        ')\n'
        ': ((x < 2) ? (\n'
        '   x + 1\n'
        ')\n'
        ': (\n'
        '   pow(x, 2)\n'
        ')))')
    assert ccode(expr, assign_to='c') == (
        'if (x < 1) {\n'
        '   c = x;\n'
        '}\n'
        'else if (x < 2) {\n'
        '   c = x + 1;\n'
        '}\n'
        'else {\n'
        '   c = pow(x, 2);\n'
        '}')
    # Check that Piecewise without a True (default) condition error
    expr = Piecewise((x, x < 1), (x**2, x > 1), (sin(x), x > 0))
    pytest.raises(ValueError, lambda: ccode(expr))


def test_ccode_Piecewise_deep():
    p = ccode(2*Piecewise((x, x < 1), (x + 1, x < 2), (x**2, True)))
    assert p == (
        '2*((x < 1) ? (\n'
        '   x\n'
        ')\n'
        ': ((x < 2) ? (\n'
        '   x + 1\n'
        ')\n'
        ': (\n'
        '   pow(x, 2)\n'
        ')))')
    expr = x*y*z + x**2 + y**2 + Piecewise((0, x < 0.5), (1, True)) + cos(z) - 1
    assert ccode(expr) == (
        'pow(x, 2) + x*y*z + pow(y, 2) + ((x < 0.5) ? (\n'
        '   0\n'
        ')\n'
        ': (\n'
        '   1\n'
        ')) + cos(z) - 1')
    assert ccode(expr, assign_to='c') == (
        'c = pow(x, 2) + x*y*z + pow(y, 2) + ((x < 0.5) ? (\n'
        '   0\n'
        ')\n'
        ': (\n'
        '   1\n'
        ')) + cos(z) - 1;')


def test_ccode_ITE():
    expr = ITE(x < 1, x, x**2)
    assert ccode(expr) == ('((x < 1) ? (\n'
                           '   x\n'
                           ')\n'
                           ': (\n'
                           '   pow(x, 2)\n'
                           '))')


def test_ccode_settings():
    pytest.raises(TypeError, lambda: ccode(sin(x), method='garbage'))


def test_ccode_Indexed():
    n, m, o = symbols('n m o', integer=True)
    i, j, k = Idx('i', n), Idx('j', m), Idx('k', o)
    p = CCodePrinter()
    p._not_c = set()

    x = IndexedBase('x')[j]
    assert p._print_Indexed(x) == 'x[j]'
    A = IndexedBase('A')[i, j]
    assert p._print_Indexed(A) == 'A[%s]' % (m*i+j)
    B = IndexedBase('B')[i, j, k]
    assert p._print_Indexed(B) == 'B[%s]' % (i*o*m+j*o+k)

    assert p._not_c == set()


def test_ccode_Indexed_without_looking_for_contraction():
    len_y = 5
    y = IndexedBase('y', shape=(len_y,))
    x = IndexedBase('x', shape=(len_y,))
    Dy = IndexedBase('Dy', shape=(len_y-1,))
    i = Idx('i', len_y - 1)
    e = Eq(Dy[i], (y[i + 1] - y[i])/(x[i + 1] - x[i]))
    code0 = ccode(e.rhs, assign_to=e.lhs, contract=False)
    assert code0 == 'Dy[i] = (y[%s] - y[i])/(x[%s] - x[i]);' % (i + 1, i + 1)


def test_ccode_loops_matrix_vector():
    n, m = symbols('n m', integer=True)
    A = IndexedBase('A')
    x = IndexedBase('x')
    y = IndexedBase('y')
    i = Idx('i', m)
    j = Idx('j', n)

    s = (
        'for (int i=0; i<m; i++){\n'
        '   y[i] = 0;\n'
        '}\n'
        'for (int i=0; i<m; i++){\n'
        '   for (int j=0; j<n; j++){\n'
        '      y[i] = x[j]*A[%s] + y[i];\n' % (i*n + j) +
        '   }\n'
        '}'
    )
    c = ccode(A[i, j]*x[j], assign_to=y[i])
    assert c == s

    pytest.raises(ValueError, lambda: ccode(A[i, j]*x[j], assign_to=x[j]))

    s2 = ('for (int i=0; i<m; i++){\n'
          '   y[i] = 0;\n'
          '}\n'
          'for (int i=0; i<m; i++){\n'
          '   for (int i=0; i<m; i++){\n'
          '      y[i] = y[i] + A[m*i + i];\n'
          '   }\n}')
    c = ccode(A[i, i], assign_to=y[i])
    assert c == s2


def test_dummy_loops():
    i, m = symbols('i m', integer=True, cls=Dummy)
    x = IndexedBase('x')
    y = IndexedBase('y')
    i = Idx(i, m)

    expected = (
        'for (int i_%(icount)i=0; i_%(icount)i<m_%(mcount)i; i_%(icount)i++){\n'
        '   y[i_%(icount)i] = x[i_%(icount)i];\n'
        '}'
    ) % {'icount': i.label.dummy_index, 'mcount': m.dummy_index}
    code = ccode(x[i], assign_to=y[i])
    assert code == expected


def test_ccode_loops_add():
    n, m = symbols('n m', integer=True)
    A = IndexedBase('A')
    x = IndexedBase('x')
    y = IndexedBase('y')
    z = IndexedBase('z')
    i = Idx('i', m)
    j = Idx('j', n)

    s = (
        'for (int i=0; i<m; i++){\n'
        '   y[i] = x[i] + z[i];\n'
        '}\n'
        'for (int i=0; i<m; i++){\n'
        '   for (int j=0; j<n; j++){\n'
        '      y[i] = x[j]*A[%s] + y[i];\n' % (i*n + j) +
        '   }\n'
        '}'
    )
    c = ccode(A[i, j]*x[j] + x[i] + z[i], assign_to=y[i])
    assert c == s


def test_ccode_loops_multiple_contractions():
    n, m, o, p = symbols('n m o p', integer=True)
    a = IndexedBase('a')
    b = IndexedBase('b')
    y = IndexedBase('y')
    i = Idx('i', m)
    j = Idx('j', n)
    k = Idx('k', o)
    l = Idx('l', p)

    s = (
        'for (int i=0; i<m; i++){\n'
        '   y[i] = 0;\n'
        '}\n'
        'for (int i=0; i<m; i++){\n'
        '   for (int j=0; j<n; j++){\n'
        '      for (int k=0; k<o; k++){\n'
        '         for (int l=0; l<p; l++){\n'
        '            y[i] = y[i] + b[%s]*a[%s];\n' % (j*o*p + k*p + l, i*n*o*p + j*o*p + k*p + l) +
        '         }\n'
        '      }\n'
        '   }\n'
        '}'
    )
    c = ccode(b[j, k, l]*a[i, j, k, l], assign_to=y[i])
    assert c == s


def test_ccode_loops_addfactor():
    n, m, o, p = symbols('n m o p', integer=True)
    a = IndexedBase('a')
    b = IndexedBase('b')
    c = IndexedBase('c')
    y = IndexedBase('y')
    i = Idx('i', m)
    j = Idx('j', n)
    k = Idx('k', o)
    l = Idx('l', p)

    s = (
        'for (int i=0; i<m; i++){\n'
        '   y[i] = 0;\n'
        '}\n'
        'for (int i=0; i<m; i++){\n'
        '   for (int j=0; j<n; j++){\n'
        '      for (int k=0; k<o; k++){\n'
        '         for (int l=0; l<p; l++){\n'
        '            y[i] = (a[%s] + b[%s])*c[%s] + y[i];\n' % (i*n*o*p + j*o*p + k*p + l, i*n*o*p + j*o*p + k*p + l, j*o*p + k*p + l) +
        '         }\n'
        '      }\n'
        '   }\n'
        '}'
    )
    c = ccode((a[i, j, k, l] + b[i, j, k, l])*c[j, k, l], assign_to=y[i])
    assert c == s


def test_ccode_loops_multiple_terms():
    n, m, o, p = symbols('n m o p', integer=True)
    a = IndexedBase('a')
    b = IndexedBase('b')
    c = IndexedBase('c')
    y = IndexedBase('y')
    i = Idx('i', m)
    j = Idx('j', n)
    k = Idx('k', o)

    s0 = (
        'for (int i=0; i<m; i++){\n'
        '   y[i] = 0;\n'
        '}\n'
    )
    s1 = (
        'for (int i=0; i<m; i++){\n'
        '   for (int j=0; j<n; j++){\n'
        '      for (int k=0; k<o; k++){\n'
        '         y[i] = b[j]*b[k]*c[%s] + y[i];\n' % (i*n*o + j*o + k) +
        '      }\n'
        '   }\n'
        '}\n'
    )
    s2 = (
        'for (int i=0; i<m; i++){\n'
        '   for (int k=0; k<o; k++){\n'
        '      y[i] = b[k]*a[%s] + y[i];\n' % (i*o + k) +
        '   }\n'
        '}\n'
    )
    s3 = (
        'for (int i=0; i<m; i++){\n'
        '   for (int j=0; j<n; j++){\n'
        '      y[i] = b[j]*a[%s] + y[i];\n' % (i*n + j) +
        '   }\n'
        '}\n'
    )
    c = ccode(
        b[j]*a[i, j] + b[k]*a[i, k] + b[j]*b[k]*c[i, j, k], assign_to=y[i])
    assert (c == s0 + s1 + s2 + s3[:-1] or
            c == s0 + s1 + s3 + s2[:-1] or
            c == s0 + s2 + s1 + s3[:-1] or
            c == s0 + s2 + s3 + s1[:-1] or
            c == s0 + s3 + s1 + s2[:-1] or
            c == s0 + s3 + s2 + s1[:-1])


def test_dereference_printing():
    expr = x + y + sin(z) + z
    assert ccode(expr, dereference=[z]) == 'x + y + (*z) + sin((*z))'


def test_Matrix_printing():
    # Test returning a Matrix
    mat = Matrix([x*y, Piecewise((2 + x, y > 0), (y, True)), sin(z)])
    A = MatrixSymbol('A', 3, 1)
    assert ccode(mat, A) == (
        'A[0] = x*y;\n'
        'if (y > 0) {\n'
        '   A[1] = x + 2;\n'
        '}\n'
        'else {\n'
        '   A[1] = y;\n'
        '}\n'
        'A[2] = sin(z);')
    # Test using MatrixElements in expressions
    expr = Piecewise((2*A[2, 0], x > 0), (A[2, 0], True)) + sin(A[1, 0]) + A[0, 0]
    assert ccode(expr) == (
        '((x > 0) ? (\n'
        '   2*A[2]\n'
        ')\n'
        ': (\n'
        '   A[2]\n'
        ')) + sin(A[1]) + A[0]')
    # Test using MatrixElements in a Matrix
    q = MatrixSymbol('q', 5, 1)
    M = MatrixSymbol('M', 3, 3)
    m = Matrix([[sin(q[1, 0]), 0, cos(q[2, 0])],
                [q[1, 0] + q[2, 0], q[3, 0], 5],
                [2*q[4, 0]/q[1, 0], sqrt(q[0, 0]) + 4, 0]])
    assert ccode(m, M) == (
        'M[0] = sin(q[1]);\n'
        'M[1] = 0;\n'
        'M[2] = cos(q[2]);\n'
        'M[3] = q[1] + q[2];\n'
        'M[4] = q[3];\n'
        'M[5] = 5;\n'
        'M[6] = 2*q[4]*1.0/q[1];\n'
        'M[7] = 4 + sqrt(q[0]);\n'
        'M[8] = 0;')


def test_ccode_reserved_words():

    y = symbols('if')

    assert ccode(y**2) == 'pow(if_, 2)'
    assert ccode(x * y**2, dereference=[y]) == 'pow((*if_), 2)*x'

    expected = 'pow(if_unreserved, 2)'
    assert ccode(y**2, reserved_word_suffix='_unreserved') == expected

    with pytest.raises(ValueError):
        ccode(y**2, error_on_reserved=True)


def test_ccode_sign():

    expr = sign(x) * y
    assert ccode(expr) == 'y*(((x) > 0) - ((x) < 0))'
    assert ccode(expr, 'z') == 'z = y*(((x) > 0) - ((x) < 0));'

    assert ccode(sign(2 * x + x**2) * x + x**2) == \
        'pow(x, 2) + x*(((pow(x, 2) + 2*x) > 0) - ((pow(x, 2) + 2*x) < 0))'

    expr = sign(cos(x))
    assert ccode(expr) == '(((cos(x)) > 0) - ((cos(x)) < 0))'


def test_mul():
    assert ccode(Mul(y, x, evaluate=False), order='none') == 'y*x'
    assert ccode(x/((x + y)*z)) == 'x/(z*(x + y))'



import pytest

from diofant import (CC, FF, CoercionFailed, FF_gmpy, FF_python,
                     PythonRational, QQ_gmpy, QQ_python, ZZ_gmpy, ZZ_python)


__all__ = ()

gmpy = pytest.importorskip('gmpy2')


def test_convert():
    assert ZZ_gmpy.finite_field(2) == FF_gmpy(2)
    assert ZZ_python.finite_field(2) == FF_python(2)

    F3 = FF(3)
    F3_gmpy = FF_gmpy(3)
    F3_python = FF_python(3)

    assert F3.convert(gmpy.mpz(2)) == F3.dtype(2)
    assert F3.convert(gmpy.mpq(2, 1)) == F3.dtype(2)
    pytest.raises(CoercionFailed, lambda: F3.convert(gmpy.mpq(1, 2)))

    assert ZZ_gmpy.convert(F3_python(1)) == ZZ_gmpy.dtype(1)
    assert ZZ_gmpy.convert(F3_gmpy(1)) == ZZ_gmpy.dtype(1)

    assert ZZ_gmpy.convert(PythonRational(2)) == ZZ_gmpy.dtype(2)
    pytest.raises(CoercionFailed,
                  lambda: ZZ_gmpy.convert(PythonRational(2, 3)))

    assert QQ_python.convert(gmpy.mpz(3)) == QQ_python.dtype(3)
    assert QQ_python.convert(gmpy.mpq(2, 3)) == QQ_python.dtype(2, 3)

    assert QQ_gmpy.convert(PythonRational(2, 3)) == QQ_gmpy.dtype(2, 3)

    assert ZZ_python.convert(F3_gmpy(1)) == ZZ_python.dtype(1)
    assert ZZ_python.convert(gmpy.mpz(3)) == ZZ_python.dtype(3)
    assert ZZ_python.convert(gmpy.mpq(3, 1)) == ZZ_python.dtype(3)
    pytest.raises(CoercionFailed,
                  lambda: ZZ_python.convert(gmpy.mpq(3, 2)))

    assert CC.convert(gmpy.mpz(3)) == CC(3)
    assert CC.convert(gmpy.mpq(1, 2)) == CC(0.5)



"""Mathematica code printing tests."""

import pytest

from diofant import (QQ, Catalan, Derivative, Dummy, E, Eq, EulerGamma,
                     Function, Gt, Heaviside, Integer, Integral, Lambda, Le,
                     Limit, Matrix, Max, Min, Ne, Or, Piecewise, Poly,
                     Rational, RootOf, RootSum, SparseMatrix, Sum, Tuple, acos,
                     acosh, acot, acoth, asin, asinh, atan, atanh, binomial,
                     conjugate, cos, cosh, cot, coth, csch, erfc, exp,
                     factorial, factorial2, false, fibonacci, gamma, hyper, im,
                     log, loggamma, mathematica_code, meijerg, oo, pi,
                     polygamma, polylog, re, rf, sech, sign, sin, sinh,
                     symbols, tan, tanh, true, zeta)
from diofant.abc import x, y, z


__all__ = ()

f = Function('f')


def test_Integer():
    assert mathematica_code(Integer(67)) == '67'
    assert mathematica_code(Integer(-1)) == '-1'


def test_Rational():
    assert mathematica_code(Rational(3, 7)) == '3/7'
    assert mathematica_code(Rational(18, 9)) == '2'
    assert mathematica_code(Rational(3, -7)) == '-3/7'
    assert mathematica_code(Rational(-3, -7)) == '3/7'
    assert mathematica_code(x + Rational(3, 7)) == 'x + 3/7'
    assert mathematica_code(Rational(3, 7)*x) == '(3/7)*x'


def test_symbols():
    assert mathematica_code(x) == 'x'
    d = Dummy('d')
    assert mathematica_code(d) == f'd{d.dummy_index}'


def test_Function():
    assert mathematica_code(f(x, y, z)) == 'f[x, y, z]'
    assert mathematica_code(sin(x) ** cos(x)) == 'Sin[x]^Cos[x]'
    assert mathematica_code(sign(x)) == 'Sign[x]'

    assert mathematica_code(atanh(x), user_functions={'atanh': 'ArcTanh'}) == 'ArcTanh[x]'

    assert (mathematica_code(meijerg(((1, 1), (3, 4)), ((1,), ()), x)) ==
            'MeijerG[{{1, 1}, {3, 4}}, {{1}, {}}, x]')
    assert (mathematica_code(hyper((1, 2, 3), (3, 4), x)) ==
            'HypergeometricPFQ[{1, 2, 3}, {3, 4}, x]')

    assert mathematica_code(Min(x, y)) == 'Min[x, y]'
    assert mathematica_code(Max(x, y)) == 'Max[x, y]'
    assert mathematica_code(Max(x, 2)) == 'Max[2, x]'  # issue sympy/sympy#15344

    assert mathematica_code(binomial(x, y)) == 'Binomial[x, y]'

    assert mathematica_code(log(x)) == 'Log[x]'
    assert mathematica_code(tan(x)) == 'Tan[x]'
    assert mathematica_code(cot(x)) == 'Cot[x]'
    assert mathematica_code(asin(x)) == 'ArcSin[x]'
    assert mathematica_code(acos(x)) == 'ArcCos[x]'
    assert mathematica_code(atan(x)) == 'ArcTan[x]'
    assert mathematica_code(acot(x)) == 'ArcCot[x]'
    assert mathematica_code(sinh(x)) == 'Sinh[x]'
    assert mathematica_code(cosh(x)) == 'Cosh[x]'
    assert mathematica_code(tanh(x)) == 'Tanh[x]'
    assert mathematica_code(coth(x)) == 'Coth[x]'
    assert mathematica_code(asinh(x)) == 'ArcSinh[x]'
    assert mathematica_code(acosh(x)) == 'ArcCosh[x]'
    assert mathematica_code(atanh(x)) == 'ArcTanh[x]'
    assert mathematica_code(acoth(x)) == 'ArcCoth[x]'
    assert mathematica_code(sech(x)) == 'Sech[x]'
    assert mathematica_code(csch(x)) == 'Csch[x]'
    assert mathematica_code(erfc(x)) == 'Erfc[x]'
    assert mathematica_code(conjugate(x)) == 'Conjugate[x]'
    assert mathematica_code(re(x)) == 'Re[x]'
    assert mathematica_code(im(x)) == 'Im[x]'
    assert mathematica_code(polygamma(x, y)) == 'PolyGamma[x, y]'
    assert mathematica_code(factorial(x)) == 'Factorial[x]'
    assert mathematica_code(factorial2(x)) == 'Factorial2[x]'
    assert mathematica_code(rf(x, y)) == 'Pochhammer[x, y]'
    assert mathematica_code(gamma(x)) == 'Gamma[x]'
    assert mathematica_code(zeta(x)) == 'Zeta[x]'
    assert mathematica_code(Heaviside(x)) == 'UnitStep[x]'
    assert mathematica_code(fibonacci(x)) == 'Fibonacci[x]'
    assert mathematica_code(polylog(x, y)) == 'PolyLog[x, y]'
    assert mathematica_code(loggamma(x)) == 'LogGamma[x]'

    class MyFunc1(Function):
        @classmethod
        def eval(cls, x):
            pass

    class MyFunc2(Function):
        @classmethod
        def eval(cls, x, y):
            pass

    pytest.raises(ValueError,
                  lambda: mathematica_code(MyFunc1(x),
                                           user_functions={'MyFunc1':
                                                           ['Myfunc1']}))
    assert mathematica_code(MyFunc1(x),
                            user_functions={'MyFunc1':
                                            'Myfunc1'}) == 'Myfunc1[x]'
    assert mathematica_code(MyFunc2(x, y),
                            user_functions={'MyFunc2':
                                            [(lambda *x: False,
                                              'Myfunc2')]}) == 'MyFunc2[x, y]'


def test_Lambda():
    f1 = Lambda(x, x**2)
    assert mathematica_code(f1) == 'Function[{x}, x^2]'
    f2 = Lambda((x, y), x + 2*y)
    assert mathematica_code(f2) == 'Function[{x, y}, x + 2*y]'


def test_Derivative():
    assert mathematica_code(Derivative(f(x), x, x)) == 'Hold[D[f[x], x, x]]'
    assert mathematica_code(Derivative(sin(x), x)) == 'Hold[D[Sin[x], x]]'
    assert mathematica_code(Derivative(x, x)) == 'Hold[D[x, x]]'
    assert mathematica_code(Derivative(sin(x)*y**4, x, 2)) == 'Hold[D[y^4*Sin[x], x, x]]'
    assert mathematica_code(Derivative(sin(x)*y**4, x, y, x)) == 'Hold[D[y^4*Sin[x], x, y, x]]'
    assert mathematica_code(Derivative(sin(x)*y**4, x, y, 3, x)) == 'Hold[D[y^4*Sin[x], x, y, y, y, x]]'


def test_Pow():
    assert mathematica_code(x**3) == 'x^3'
    assert mathematica_code(x**(y**3)) == 'x^(y^3)'
    assert mathematica_code(1/(f(x)*3.5)**(x - y**x)/(x**2 + y)) == \
        '(3.5*f[x])^(-x + y^x)/(x^2 + y)'
    assert mathematica_code(x**-1.0) == 'x^(-1.0)'
    assert mathematica_code(x**Rational(2, 3)) == 'x^(2/3)'


def test_Mul():
    A, B, C, D = symbols('A B C D', commutative=False)
    assert mathematica_code(x*y*z) == 'x*y*z'
    assert mathematica_code(x*y*A) == 'x*y*A'
    assert mathematica_code(x*y*A*B) == 'x*y*A**B'
    assert mathematica_code(x*y*A*B*C) == 'x*y*A**B**C'
    assert mathematica_code(x*A*B*(C + D)*A*y) == 'x*y*A**B**(C + D)**A'


def test_constants():
    assert mathematica_code(pi) == 'Pi'
    assert mathematica_code(oo) == 'Infinity'
    assert mathematica_code(-oo) == '-Infinity'
    assert mathematica_code(EulerGamma) == 'EulerGamma'
    assert mathematica_code(Catalan) == 'Catalan'
    assert mathematica_code(E) == 'E'


def test_containers():
    assert mathematica_code([1, 2, 3, [4, 5, [6, 7]], 8, [9, 10], 11]) == \
        '{1, 2, 3, {4, 5, {6, 7}}, 8, {9, 10}, 11}'
    assert mathematica_code((1, 2, (3, 4))) == '{1, 2, {3, 4}}'
    assert mathematica_code([1]) == '{1}'
    assert mathematica_code((1,)) == '{1}'
    assert mathematica_code(Tuple(*[1, 2, 3])) == '{1, 2, 3}'


def test_Integral():
    assert mathematica_code(Integral(sin(sin(x)), x)) == 'Hold[Integrate[Sin[Sin[x]], x]]'
    assert mathematica_code(Integral(exp(-x**2 - y**2),
                                     (x, -oo, oo),
                                     (y, -oo, oo))) == \
        'Hold[Integrate[E^(-x^2 - y^2), {x, -Infinity, Infinity}, ' \
        '{y, -Infinity, Infinity}]]'


def test_Sum():
    assert mathematica_code(Sum(sin(x), (x, 0, 10))) == 'Hold[Sum[Sin[x], {x, 0, 10}]]'
    assert mathematica_code(Sum(exp(-x**2 - y**2),
                                (x, -oo, oo),
                                (y, -oo, oo))) == \
        'Hold[Sum[E^(-x^2 - y^2), {x, -Infinity, Infinity}, ' \
        '{y, -Infinity, Infinity}]]'


def test_Matrix():
    assert mathematica_code(Matrix()) == '{}'

    m = Matrix([[1, 2], [3, 4444]])
    assert mathematica_code(m) == mathematica_code(m.as_immutable()) == '{{1, 2}, {3, 4444}}'

    m = SparseMatrix(m)
    assert mathematica_code(m) == mathematica_code(m.as_immutable()) == '{{1, 2}, {3, 4444}}'


def test_Relational():
    assert mathematica_code(Eq(x, y)) == 'x == y'
    assert mathematica_code(Ne(x, y/(1 + y**2))) == 'x != (y/(y^2 + 1))'
    assert mathematica_code(Le(0, x**2)) == '0 <= x^2'
    assert mathematica_code(Gt(pi, 3, evaluate=False)) == 'Pi > 3'


def test_Booleans():
    assert mathematica_code(true) == 'True'
    assert mathematica_code(false) == 'False'


def test_Piecewise():
    g = Piecewise((0, Or(x <= -1, x >= 1)), (1 - x, x > 0), (1 + x, True))

    assert (mathematica_code(g) ==
            'Piecewise[{{0, x >= 1 || x <= -1}, '
            '{-x + 1, x > 0}, {x + 1, True}}]')


def test_RootOf():
    p = Poly(x**3 + y*x + 1, x)
    assert mathematica_code(RootOf(p, 0)) == 'Root[#^3 + #*y + 1 &, 1]'


def test_RootSum():
    r = RootSum(x**3 + x + 3, Lambda(y, log(y*z)))
    assert mathematica_code(r) == ('RootSum[Function[{x}, x^3 + x + 3], '
                                   'Function[{y}, Log[y*z]]]')


def test_AlgebraicElement():
    r = RootOf(x**7 + 3*x - 1, 3)
    K = QQ.algebraic_field(r)
    a = K([1, 2, 3, 0, 1])
    assert mathematica_code(a) == ('AlgebraicNumber[Root[#^7 + 3*# - 1 &, 4],'
                                   ' {1, 0, 3, 2, 1}]')


def test_Limit():
    e = Limit(sin(x)/x, x, 0)
    assert mathematica_code(e) == 'Hold[Limit[Sin[x]/x, x -> 0, Direction -> -1]]'
    e = Limit(sin(x)/x, x, 0, '-')
    assert mathematica_code(e) == 'Hold[Limit[Sin[x]/x, x -> 0, Direction -> 1]]'
    e = Limit(sin(x)/x, x, 0, 'real')
    assert mathematica_code(e) == 'Hold[Limit[Sin[x]/x, x -> 0, Direction -> Reals]]'



import functools

from ..utilities import default_sort_key
from .str import StrPrinter


class LambdaPrinter(StrPrinter):
    """
    This printer converts expressions into strings that can be used by
    lambdify.
    """

    def _print_MatrixBase(self, expr):
        return f'{expr.__class__.__name__}({self._print(expr.tolist())})'

    _print_SparseMatrix = \
        _print_MutableSparseMatrix = \
        _print_ImmutableSparseMatrix = \
        _print_Matrix = \
        _print_DenseMatrix = \
        _print_MutableDenseMatrix = \
        _print_ImmutableMatrix = \
        _print_ImmutableDenseMatrix = \
        _print_MatrixBase

    def _print_Piecewise(self, expr):
        result = []
        i = 0
        for arg in expr.args:
            e = arg.expr
            c = arg.cond
            result.append('((')
            result.append(self._print(e))
            result.append(') if (')
            result.append(self._print(c))
            result.append(') else (')
            i += 1
        result = result[:-1]
        result.append(') else None)')
        result.append(')'*(2*i - 2))
        return ''.join(result)

    def _print_And(self, expr):
        result = ['(']
        for arg in sorted(expr.args, key=default_sort_key):
            result.extend(['(', self._print(arg), ')'])
            result.append(' and ')
        result = result[:-1]
        result.append(')')
        return ''.join(result)

    def _print_Or(self, expr):
        result = ['(']
        for arg in sorted(expr.args, key=default_sort_key):
            result.extend(['(', self._print(arg), ')'])
            result.append(' or ')
        result = result[:-1]
        result.append(')')
        return ''.join(result)

    def _print_Not(self, expr):
        result = ['(', 'not (', self._print(expr.args[0]), '))']
        return ''.join(result)

    def _print_BooleanTrue(self, expr):
        return 'True'

    def _print_BooleanFalse(self, expr):
        return 'False'

    def _print_ITE(self, expr):
        result = [
            '((', self._print(expr.args[1]),
            ') if (', self._print(expr.args[0]),
            ') else (', self._print(expr.args[2]), '))'
        ]
        return ''.join(result)

    def _print_Dummy(self, expr):
        return super()._print_Dummy(expr).replace('(', '_lpar_').replace(')', '_rpar_')


class NumPyPrinter(LambdaPrinter):
    """
    Numpy printer which handles vectorized piecewise functions,
    logical operators, etc.
    """

    _default_settings = {
        'order': 'none',
        'full_prec': 'auto',
    }

    def _print_MatMul(self, expr):
        """Matrix multiplication printer"""
        return f"({').dot('.join(self._print(i) for i in expr.args)})"

    def _print_Piecewise(self, expr):
        """Piecewise function printer"""
        exprs = f"[{','.join(self._print(arg.expr) for arg in expr.args)}]"
        conds = f"[{','.join(self._print(arg.cond) for arg in expr.args)}]"
        # If [default_value, True] is a (expr, cond) sequence in a Piecewise object
        #     it will behave the same as passing the 'default' kwarg to select()
        #     *as long as* it is the last element in expr.args.
        # If this is not the case, it may be triggered prematurely.
        return f'select({conds}, {exprs}, default=nan)'

    def _print_Relational(self, expr):
        """Relational printer"""
        op = {'==': 'equal',
              '!=': 'not_equal',
              '<': 'less',
              '<=': 'less_equal',
              '>': 'greater',
              '>=': 'greater_equal'}
        return f'{op[expr.rel_op]}({self._print(expr.lhs)}, {self._print(expr.rhs)})'

    def _print_And(self, expr):
        """Logical And printer"""
        # We have to override LambdaPrinter because it uses Python 'and' keyword.
        # If LambdaPrinter didn't define it, we could use StrPrinter's
        # version of the function and add 'logical_and' to NUMPY_TRANSLATIONS.
        return functools.reduce(lambda x, y: f'logical_and({self._print(x)}, {self._print(y)})', expr.args)

    def _print_Or(self, expr):
        """Logical Or printer"""
        # We have to override LambdaPrinter because it uses Python 'or' keyword.
        # If LambdaPrinter didn't define it, we could use StrPrinter's
        # version of the function and add 'logical_or' to NUMPY_TRANSLATIONS.
        return functools.reduce(lambda x, y: f'logical_or({self._print(x)}, {self._print(y)})', expr.args)

    def _print_Xor(self, expr):
        """Logical Xor printer"""
        return functools.reduce(lambda x, y: f'logical_xor({self._print(x)}, {self._print(y)})', expr.args)

    def _print_Not(self, expr):
        """Logical Not printer"""
        # We have to override LambdaPrinter because it uses Python 'not' keyword.
        # If LambdaPrinter didn't define it, we would still have to define our
        #     own because StrPrinter doesn't define it.
        return f"{'logical_not'}({','.join(self._print(i) for i in expr.args)})"

    def _print_Min(self, expr):
        return f"{'amin'}(({','.join(self._print(i) for i in expr.args)}))"

    def _print_Max(self, expr):
        return f"{'amax'}(({','.join(self._print(i) for i in expr.args)}))"


class MpmathPrinter(LambdaPrinter):
    """Mpmath printer."""

    def _print_RootOf(self, expr):
        if expr.is_real:
            return ('findroot(lambda %s: %s, %s, '
                    "method='bisection')" % (self._print(expr.poly.gen),
                                             self._print(expr.expr),
                                             self._print(expr.interval.as_tuple())))
        else:
            return ('findroot(lambda %s: %s, mpc%s, '
                    "method='secant')" % (self._print(expr.poly.gen),
                                          self._print(expr.expr),
                                          self._print(expr.interval.center)))

    def _print_Sum(self, expr):
        return 'nsum(lambda %s: %s, %s)' % (','.join([self._print(v) for v in expr.variables]),
                                            self._print(expr.function),
                                            ','.join([self._print(v[1:]) for v in expr.limits]))

    def _print_Infinity(self, expr):
        return 'inf'

    def _print_Float(self, e):
        # XXX: This does not handle setting mpmath.mp.dps. It is assumed that
        # the caller of the lambdified function will have set it to sufficient
        # precision to match the Floats in the expression.

        # Remove 'mpz' if gmpy is installed.
        args = str(tuple(map(int, e._mpf_)))
        return f'mpf({args})'

    def _print_GoldenRatio(self, expr):
        return 'phi'

    def _print_Pow(self, expr):
        if expr.exp.is_Rational:
            n, d = expr.exp.as_numer_denom()
            if d == 1:
                if n >= 0:
                    return f'{self._print(expr.base)}**{n}'
                else:
                    return f'power({self._print(expr.base)}, {n})'
            else:
                if n >= 2:
                    return f'root({self._print(expr.base)}, {d})**{n}'
                elif n == 1:
                    return f'root({self._print(expr.base)}, {d})'
                else:
                    return f'power(root({self._print(expr.base)}, {d}), {n})'
        else:
            return super()._print_Pow(expr)

    def _print_Rational(self, expr):
        n, d = expr.numerator, expr.denominator
        if d == 1:
            return f'{n}'
        else:
            return f'{n}*power({d}, -1)'


def lambdarepr(expr, **settings):
    """
    Returns a string usable for lambdifying.
    """
    return LambdaPrinter(settings).doprint(expr)



from diofant import QQ, GoldenRatio, Rational, RootOf, Sum, oo, pi, sqrt
from diofant.abc import n, x
from diofant.printing.lambdarepr import MpmathPrinter


__all__ = ()


def test_basic():
    p = MpmathPrinter()

    assert p.doprint(GoldenRatio) == 'phi'
    assert p.doprint(Rational(2)) == '2'
    assert p.doprint(Rational(2, 3)) == '2*power(3, -1)'


def test_Pow():
    p = MpmathPrinter()

    assert p.doprint(sqrt(pi)) == 'root(pi, 2)'
    assert p.doprint(pi**Rational(2, 3)) == 'root(pi, 3)**2'
    assert p.doprint(pi**Rational(-2, 3)) == 'power(root(pi, 3), -2)'
    assert p.doprint(pi**pi) == 'pi**pi'


def test_RootOf():
    p = MpmathPrinter()

    e = RootOf(x**3 + x - 1, x, 0)
    r = ('findroot(lambda x: x**3 + x - 1, (%s, %s), '
         "method='bisection')" % (p.doprint(QQ(0)), p.doprint(QQ(1))))

    assert p.doprint(e) == r

    e = RootOf(x**3 + x - 1, x, 1)
    r = ('findroot(lambda x: x**3 + x - 1, mpc(%s, %s), '
         "method='secant')" % (p.doprint(QQ(-3, 8)), p.doprint(QQ(-9, 8))))

    assert p.doprint(e) == r


def test_Sum():
    p = MpmathPrinter()

    s = Sum(n**(-2), (n, 1, oo))

    assert p.doprint(s) == 'nsum(lambda n: power(n, -2), (1, inf))'



from diofant import exp, log, sign
from diofant.abc import x
from diofant.calculus.singularities import singularities


__all__ = ()


def test_singularities():
    assert singularities(x**2, x) == set()
    assert singularities(x/(x**2 + 3*x + 2), x) == {-2, -1}
    assert singularities(1/(1 + x), x) == {-1}
    assert singularities(sign(x), x) == {0}


def test_singularities_non_rational():
    assert singularities(exp(1/x), x) == {0}
    assert singularities(log((x - 2)**2), x) == {2}
    assert singularities(exp(1/x) + log(x + 1), x) == {-1, 0}
    assert singularities(exp(1/log(x + 1)), x) == {0}
